@@ -33,10 +33,11 @@ import java.nio.file.Path;
 import java.nio.file.attribute.BasicFileAttributes;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
+import java.util.Comparator;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
-import org.apache.commons.io.FileUtils;
+import java.util.stream.Stream;
 import org.apache.directory.api.util.Hex;
 import org.apache.iceberg.relocated.com.google.common.base.Preconditions;
 import org.apache.iceberg.relocated.com.google.common.collect.Lists;
@@ -101,7 +102,7 @@ public class AliyunOSSMockLocalStore {
 
   void createBucket(String bucketName) throws IOException {
     File newBucket = new File(root, bucketName);
-    FileUtils.forceMkdir(newBucket);
+    Files.createDirectory(newBucket.toPath());
   }
 
   Bucket getBucket(String bucketName) {
@@ -122,7 +123,9 @@ public class AliyunOSSMockLocalStore {
           409, OSSErrorCode.BUCKET_NOT_EMPTY, "The bucket you tried to delete is not empty. ");
     }
 
-    FileUtils.deleteDirectory(dir);
+    try (Stream<Path> walk = Files.walk(dir.toPath())) {
+      walk.sorted(Comparator.reverseOrder()).map(Path::toFile).forEach(File::delete);
+    }
   }
 
   ObjectMetadata putObject(
