@@ -257,6 +257,11 @@ public class SerializableTable implements Table, Serializable {
     return lazyTable().newScan();
   }
 
+  @Override
+  public BatchScan newBatchScan() {
+    return lazyTable().newBatchScan();
+  }
+
   @Override
   public Snapshot currentSnapshot() {
     return lazyTable().currentSnapshot();
@@ -375,6 +380,10 @@ public class SerializableTable implements Table, Serializable {
     protected Table newTable(TableOperations ops, String tableName) {
       return MetadataTableUtils.createMetadataTableInstance(ops, baseTableName, tableName, type);
     }
+
+    public MetadataTableType type() {
+      return type;
+    }
   }
 
   // captures the current state of a Hadoop configuration in a serializable manner
