@@ -39,8 +39,13 @@ import org.apache.iceberg.types.Types;
 import org.apache.iceberg.util.Filter;
 import org.apache.iceberg.util.SortedMerge;
 import org.apache.iceberg.util.StructLikeSet;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 public class Deletes {
+
+  private static final Logger LOG = LoggerFactory.getLogger(Deletes.class);
+
   private static final Schema POSITION_DELETE_SCHEMA =
       new Schema(MetadataColumns.DELETE_FILE_PATH, MetadataColumns.DELETE_FILE_POS);
 
@@ -219,7 +224,7 @@ public class Deletes {
       CloseableIterator<T> iter;
       if (deletePosIterator.hasNext()) {
         nextDeletePos = deletePosIterator.next();
-        iter = applyDelete(rows.iterator());
+        iter = applyDelete(rows.iterator(), deletePosIterator);
       } else {
         iter = rows.iterator();
       }
@@ -249,7 +254,8 @@ public class Deletes {
       return isDeleted;
     }
 
-    protected abstract CloseableIterator<T> applyDelete(CloseableIterator<T> items);
+    protected abstract CloseableIterator<T> applyDelete(
+        CloseableIterator<T> items, CloseableIterator<Long> deletePositions);
   }
 
   private static class PositionStreamDeleteFilter<T> extends PositionStreamDeleteIterable<T> {
@@ -265,7 +271,8 @@ public class Deletes {
     }
 
     @Override
-    protected CloseableIterator<T> applyDelete(CloseableIterator<T> items) {
+    protected CloseableIterator<T> applyDelete(
+        CloseableIterator<T> items, CloseableIterator<Long> deletePositions) {
       return new FilterIterator<T>(items) {
         @Override
         protected boolean shouldKeep(T item) {
@@ -276,6 +283,16 @@ public class Deletes {
 
           return !deleted;
         }
+
+        @Override
+        public void close() {
+          try {
+            deletePositions.close();
+          } catch (IOException e) {
+            LOG.warn("Error closing delete file", e);
+          }
+          super.close();
+        }
       };
     }
   }
@@ -293,15 +310,38 @@ public class Deletes {
     }
 
     @Override
-    protected CloseableIterator<T> applyDelete(CloseableIterator<T> items) {
-      return CloseableIterator.transform(
-          items,
-          row -> {
-            if (isDeleted(row)) {
-              markDeleted.accept(row);
-            }
-            return row;
-          });
+    protected CloseableIterator<T> applyDelete(
+        CloseableIterator<T> items, CloseableIterator<Long> deletePositions) {
+
+      return new CloseableIterator<T>() {
+        @Override
+        public void close() {
+          try {
+            deletePositions.close();
+          } catch (IOException e) {
+            LOG.warn("Error closing delete file", e);
+          }
+          try {
+            items.close();
+          } catch (IOException e) {
+            LOG.warn("Error closing data file", e);
+          }
+        }
+
+        @Override
+        public boolean hasNext() {
+          return items.hasNext();
+        }
+
+        @Override
+        public T next() {
+          T row = items.next();
+          if (isDeleted(row)) {
+            markDeleted.accept(row);
+          }
+          return row;
+        }
+      };
     }
   }
 
