@@ -58,12 +58,12 @@ public class TableMigrationUtil {
    * Returns the data files in a partition by listing the partition location.
    *
    * <p>For Parquet and ORC partitions, this will read metrics from the file footer. For Avro
-   * partitions, metrics are set to null.
+   * partitions, metrics other than row count are set to null.
    *
-   * <p>Note: certain metrics, like NaN counts, that are only supported by iceberg file writers but
+   * <p>Note: certain metrics, like NaN counts, that are only supported by Iceberg file writers but
    * not file footers, will not be populated.
    *
-   * @param partition partition key, e.g., "a=1/b=2"
+   * @param partition map of column names to column values for the partition
    * @param uri partition location URI
    * @param format partition format, avro, parquet or orc
    * @param spec a partition spec
@@ -83,8 +83,28 @@ public class TableMigrationUtil {
     return listPartition(partition, uri, format, spec, conf, metricsConfig, mapping, 1);
   }
 
+  /**
+   * Returns the data files in a partition by listing the partition location. Metrics are read from
+   * the files and the file reading is done in parallel by a specified number of threads.
+   *
+   * <p>For Parquet and ORC partitions, this will read metrics from the file footer. For Avro
+   * partitions, metrics other than row count are set to null.
+   *
+   * <p>Note: certain metrics, like NaN counts, that are only supported by Iceberg file writers but
+   * not file footers, will not be populated.
+   *
+   * @param partition map of column names to column values for the partition
+   * @param partitionUri partition location URI
+   * @param format partition format, avro, parquet or orc
+   * @param spec a partition spec
+   * @param conf a Hadoop conf
+   * @param metricsSpec a metrics conf
+   * @param mapping a name mapping
+   * @param parallelism number of threads to use for file reading
+   * @return a List of DataFile
+   */
   public static List<DataFile> listPartition(
-      Map<String, String> partitionPath,
+      Map<String, String> partition,
       String partitionUri,
       String format,
       PartitionSpec spec,
@@ -94,16 +114,16 @@ public class TableMigrationUtil {
       int parallelism) {
     ExecutorService service = null;
     try {
-      String partitionKey =
+      List<String> partitionValues =
           spec.fields().stream()
               .map(PartitionField::name)
-              .map(name -> String.format("%s=%s", name, partitionPath.get(name)))
-              .collect(Collectors.joining("/"));
+              .map(partition::get)
+              .collect(Collectors.toList());
 
-      Path partition = new Path(partitionUri);
-      FileSystem fs = partition.getFileSystem(conf);
+      Path partitionDir = new Path(partitionUri);
+      FileSystem fs = partitionDir.getFileSystem(conf);
       List<FileStatus> fileStatus =
-          Arrays.stream(fs.listStatus(partition, HIDDEN_PATH_FILTER))
+          Arrays.stream(fs.listStatus(partitionDir, HIDDEN_PATH_FILTER))
               .filter(FileStatus::isFile)
               .collect(Collectors.toList());
       DataFile[] datafiles = new DataFile[fileStatus.size()];
@@ -120,7 +140,7 @@ public class TableMigrationUtil {
             index -> {
               Metrics metrics = getAvroMetrics(fileStatus.get(index).getPath(), conf);
               datafiles[index] =
-                  buildDataFile(fileStatus.get(index), partitionKey, spec, metrics, "avro");
+                  buildDataFile(fileStatus.get(index), partitionValues, spec, metrics, "avro");
             });
       } else if (format.contains("parquet")) {
         task.run(
@@ -128,7 +148,7 @@ public class TableMigrationUtil {
               Metrics metrics =
                   getParquetMetrics(fileStatus.get(index).getPath(), conf, metricsSpec, mapping);
               datafiles[index] =
-                  buildDataFile(fileStatus.get(index), partitionKey, spec, metrics, "parquet");
+                  buildDataFile(fileStatus.get(index), partitionValues, spec, metrics, "parquet");
             });
       } else if (format.contains("orc")) {
         task.run(
@@ -136,7 +156,7 @@ public class TableMigrationUtil {
               Metrics metrics =
                   getOrcMetrics(fileStatus.get(index).getPath(), conf, metricsSpec, mapping);
               datafiles[index] =
-                  buildDataFile(fileStatus.get(index), partitionKey, spec, metrics, "orc");
+                  buildDataFile(fileStatus.get(index), partitionValues, spec, metrics, "orc");
             });
       } else {
         throw new UnsupportedOperationException("Unknown partition format: " + format);
@@ -181,13 +201,17 @@ public class TableMigrationUtil {
   }
 
   private static DataFile buildDataFile(
-      FileStatus stat, String partitionKey, PartitionSpec spec, Metrics metrics, String format) {
+      FileStatus stat,
+      List<String> partitionValues,
+      PartitionSpec spec,
+      Metrics metrics,
+      String format) {
     return DataFiles.builder(spec)
         .withPath(stat.getPath().toString())
         .withFormat(format)
         .withFileSizeInBytes(stat.getLen())
         .withMetrics(metrics)
-        .withPartitionPath(partitionKey)
+        .withPartitionValues(partitionValues)
         .build();
   }
 
