@@ -18,12 +18,13 @@
  */
 package org.apache.iceberg.dell.ecs;
 
+import static org.assertj.core.api.Assertions.assertThat;
+
 import com.emc.object.s3.request.PutObjectRequest;
 import java.io.IOException;
 import java.nio.charset.StandardCharsets;
 import org.apache.iceberg.dell.mock.ecs.EcsS3MockRule;
 import org.apache.iceberg.metrics.MetricsContext;
-import org.junit.Assert;
 import org.junit.ClassRule;
 import org.junit.Test;
 
@@ -41,7 +42,7 @@ public class TestEcsSeekableInputStream {
         new EcsSeekableInputStream(
             rule.client(), new EcsURI(rule.bucket(), objectName), MetricsContext.nullMetrics())) {
       input.seek(2);
-      Assert.assertEquals("Expect 2 when seek to 2", '2', input.read());
+      assertThat(input.read()).as("Expect 2 when seek to 2").isEqualTo('2');
     }
   }
 
@@ -56,7 +57,7 @@ public class TestEcsSeekableInputStream {
             rule.client(), new EcsURI(rule.bucket(), objectName), MetricsContext.nullMetrics())) {
       input.seek(999);
       input.seek(3);
-      Assert.assertEquals("Expect 3 when seek to 3 finally", '3', input.read());
+      assertThat(input.read()).as("Expect 3 when seek to 3 finally").isEqualTo('3');
     }
   }
 
@@ -69,7 +70,7 @@ public class TestEcsSeekableInputStream {
     try (EcsSeekableInputStream input =
         new EcsSeekableInputStream(
             rule.client(), new EcsURI(rule.bucket(), objectName), MetricsContext.nullMetrics())) {
-      Assert.assertEquals("The first byte should be 0 ", '0', input.read());
+      assertThat(input.read()).as("The first byte should be 0 ").isEqualTo('0');
     }
   }
 
@@ -83,9 +84,10 @@ public class TestEcsSeekableInputStream {
         new EcsSeekableInputStream(
             rule.client(), new EcsURI(rule.bucket(), objectName), MetricsContext.nullMetrics())) {
       byte[] buffer = new byte[3];
-      Assert.assertEquals("The first read should be 3 bytes", 3, input.read(buffer));
-      Assert.assertEquals(
-          "The first 3 bytes should be 012", "012", new String(buffer, StandardCharsets.UTF_8));
+      assertThat(input.read(buffer)).as("The first read should be 3 bytes").isEqualTo(3);
+      assertThat(new String(buffer, StandardCharsets.UTF_8))
+          .as("The first 3 bytes should be 012")
+          .isEqualTo("012");
     }
   }
 }
