@@ -18,6 +18,8 @@
  */
 package org.apache.iceberg.dell.mock.ecs;
 
+import static org.assertj.core.api.Assertions.assertThat;
+
 import com.emc.object.s3.S3Client;
 import com.emc.object.s3.bean.ListObjectsResult;
 import com.emc.object.s3.bean.ObjectKey;
@@ -31,7 +33,6 @@ import java.util.stream.Collectors;
 import org.apache.iceberg.dell.DellClientFactories;
 import org.apache.iceberg.dell.DellProperties;
 import org.apache.iceberg.dell.mock.MockDellClientFactory;
-import org.junit.Assert;
 import org.junit.rules.TestRule;
 import org.junit.runner.Description;
 import org.junit.runners.model.Statement;
@@ -71,7 +72,7 @@ public class EcsS3MockRule implements TestRule {
   /** Load rule from thread local and check bucket */
   public static EcsS3MockRule rule(String id) {
     EcsS3MockRule rule = TEST_RULE_FOR_MOCK_CLIENT.get();
-    Assert.assertTrue("Test Rule must match id", rule != null && rule.bucket().equals(id));
+    assertThat(rule).isNotNull().extracting(EcsS3MockRule::bucket).isEqualTo(id);
     return rule;
   }
 
