@@ -18,6 +18,8 @@
  */
 package org.apache.iceberg.hive;
 
+import static org.assertj.core.api.Assertions.assertThat;
+import static org.assertj.core.api.Assertions.assertThatThrownBy;
 import static org.mockito.Mockito.any;
 import static org.mockito.Mockito.atLeastOnce;
 import static org.mockito.Mockito.doAnswer;
@@ -25,6 +27,8 @@ import static org.mockito.Mockito.doNothing;
 import static org.mockito.Mockito.doReturn;
 import static org.mockito.Mockito.doThrow;
 import static org.mockito.Mockito.eq;
+import static org.mockito.Mockito.mock;
+import static org.mockito.Mockito.mockStatic;
 import static org.mockito.Mockito.never;
 import static org.mockito.Mockito.reset;
 import static org.mockito.Mockito.spy;
@@ -58,14 +62,13 @@ import org.apache.iceberg.relocated.com.google.common.collect.ImmutableMap;
 import org.apache.iceberg.relocated.com.google.common.collect.Lists;
 import org.apache.iceberg.types.Types;
 import org.apache.thrift.TException;
-import org.assertj.core.api.Assertions;
-import org.junit.AfterClass;
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.BeforeClass;
-import org.junit.Test;
+import org.junit.jupiter.api.AfterAll;
+import org.junit.jupiter.api.BeforeAll;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
 import org.mockito.AdditionalAnswers;
 import org.mockito.ArgumentCaptor;
+import org.mockito.MockedStatic;
 import org.mockito.invocation.InvocationOnMock;
 
 public class TestHiveCommitLocks extends HiveTableBaseTest {
@@ -85,7 +88,7 @@ public class TestHiveCommitLocks extends HiveTableBaseTest {
   LockResponse notAcquiredLockResponse = new LockResponse(dummyLockId, LockState.NOT_ACQUIRED);
   ShowLocksResponse emptyLocks = new ShowLocksResponse(Lists.newArrayList());
 
-  @BeforeClass
+  @BeforeAll
   public static void startMetastore() throws Exception {
     HiveMetastoreTest.startMetastore(
         ImmutableMap.of(HiveConf.ConfVars.HIVE_TXN_TIMEOUT.varname, "1s"));
@@ -114,12 +117,12 @@ public class TestHiveCommitLocks extends HiveTableBaseTest {
     spyCachedClientPool = spy(new CachedClientPool(hiveConf, Collections.emptyMap()));
     when(spyCachedClientPool.clientPool()).thenAnswer(invocation -> spyClientPool);
 
-    Assert.assertNotNull(spyClientRef.get());
+    assertThat(spyClientRef.get()).isNotNull();
 
     spyClient = spyClientRef.get();
   }
 
-  @Before
+  @BeforeEach
   public void before() throws Exception {
     Table table = catalog.loadTable(TABLE_IDENTIFIER);
     ops = (HiveTableOperations) ((HasTableOperations) table).operations();
@@ -134,7 +137,7 @@ public class TestHiveCommitLocks extends HiveTableBaseTest {
 
     metadataV2 = ops.current();
 
-    Assert.assertEquals(2, ops.current().schema().columns().size());
+    assertThat(ops.current().schema().columns()).hasSize(2);
 
     spyOps =
         spy(
@@ -148,7 +151,7 @@ public class TestHiveCommitLocks extends HiveTableBaseTest {
     reset(spyClient);
   }
 
-  @AfterClass
+  @AfterAll
   public static void cleanup() {
     try {
       spyClientPool.close();
@@ -165,7 +168,7 @@ public class TestHiveCommitLocks extends HiveTableBaseTest {
 
     spyOps.doCommit(metadataV2, metadataV1);
 
-    Assert.assertEquals(1, spyOps.current().schema().columns().size()); // should be 1 again
+    assertThat(spyOps.current().schema().columns()).hasSize(1); // should be 1 again
   }
 
   @Test
@@ -184,7 +187,7 @@ public class TestHiveCommitLocks extends HiveTableBaseTest {
 
     spyOps.doCommit(metadataV2, metadataV1);
 
-    Assert.assertEquals(1, spyOps.current().schema().columns().size()); // should be 1 again
+    assertThat(spyOps.current().schema().columns()).hasSize(1); // should be 1 again
   }
 
   @Test
@@ -202,7 +205,8 @@ public class TestHiveCommitLocks extends HiveTableBaseTest {
 
     spyOps.doCommit(metadataV2, metadataV1);
 
-    Assert.assertEquals(1, spyOps.current().schema().columns().size()); // should be 1 again
+    verify(spyClient, times(1)).showLocks(any()); // Make sure HiveLock's findLock method is called
+    assertThat(spyOps.current().schema().columns()).hasSize(1); // should be 1 again
   }
 
   @Test
@@ -226,7 +230,8 @@ public class TestHiveCommitLocks extends HiveTableBaseTest {
 
     spyOps.doCommit(metadataV2, metadataV1);
 
-    Assert.assertEquals(1, spyOps.current().schema().columns().size()); // should be 1 again
+    verify(spyClient, times(1)).showLocks(any()); // Make sure HiveLock's findLock method is called
+    assertThat(spyOps.current().schema().columns()).hasSize(1); // should be 1 again
   }
 
   @Test
@@ -280,7 +285,7 @@ public class TestHiveCommitLocks extends HiveTableBaseTest {
     doNothing().when(spyClient).unlock(eq(dummyLockId));
     doNothing().when(spyClient).heartbeat(eq(0L), eq(dummyLockId));
 
-    Assertions.assertThatThrownBy(() -> spyOps.doCommit(metadataV2, metadataV1))
+    assertThatThrownBy(() -> spyOps.doCommit(metadataV2, metadataV1))
         .isInstanceOf(RuntimeException.class)
         .hasMessage(
             "org.apache.iceberg.hive.LockException: "
@@ -304,7 +309,7 @@ public class TestHiveCommitLocks extends HiveTableBaseTest {
     doNothing().when(spyClient).unlock(eq(dummyLockId));
     doNothing().when(spyClient).heartbeat(eq(0L), eq(dummyLockId));
 
-    Assertions.assertThatThrownBy(() -> spyOps.doCommit(metadataV2, metadataV1))
+    assertThatThrownBy(() -> spyOps.doCommit(metadataV2, metadataV1))
         .isInstanceOf(RuntimeException.class)
         .hasMessage(
             "org.apache.iceberg.hive.LockException: "
@@ -328,7 +333,7 @@ public class TestHiveCommitLocks extends HiveTableBaseTest {
     doNothing().when(spyClient).unlock(eq(dummyLockId));
     doNothing().when(spyClient).heartbeat(eq(0L), eq(dummyLockId));
 
-    Assertions.assertThatThrownBy(() -> spyOps.doCommit(metadataV2, metadataV1))
+    assertThatThrownBy(() -> spyOps.doCommit(metadataV2, metadataV1))
         .isInstanceOf(RuntimeException.class)
         .hasMessage("Interrupted during commit");
 
@@ -363,7 +368,7 @@ public class TestHiveCommitLocks extends HiveTableBaseTest {
   public void testLockFailureAtFirstTime() throws TException {
     doReturn(notAcquiredLockResponse).when(spyClient).lock(any());
 
-    Assertions.assertThatThrownBy(() -> spyOps.doCommit(metadataV2, metadataV1))
+    assertThatThrownBy(() -> spyOps.doCommit(metadataV2, metadataV1))
         .isInstanceOf(CommitFailedException.class)
         .hasMessage(
             "org.apache.iceberg.hive.LockException: "
@@ -381,7 +386,7 @@ public class TestHiveCommitLocks extends HiveTableBaseTest {
         .when(spyClient)
         .checkLock(eq(dummyLockId));
 
-    Assertions.assertThatThrownBy(() -> spyOps.doCommit(metadataV2, metadataV1))
+    assertThatThrownBy(() -> spyOps.doCommit(metadataV2, metadataV1))
         .isInstanceOf(CommitFailedException.class)
         .hasMessage(
             "org.apache.iceberg.hive.LockException: "
@@ -393,7 +398,7 @@ public class TestHiveCommitLocks extends HiveTableBaseTest {
     doReturn(waitLockResponse).when(spyClient).lock(any());
     doReturn(waitLockResponse).when(spyClient).checkLock(eq(dummyLockId));
 
-    Assertions.assertThatThrownBy(() -> spyOps.doCommit(metadataV2, metadataV1))
+    assertThatThrownBy(() -> spyOps.doCommit(metadataV2, metadataV1))
         .isInstanceOf(CommitFailedException.class)
         .hasMessageStartingWith("org.apache.iceberg.hive.LockException")
         .hasMessageContaining("Timed out after")
@@ -408,12 +413,38 @@ public class TestHiveCommitLocks extends HiveTableBaseTest {
         .when(spyClient)
         .checkLock(eq(dummyLockId));
 
-    Assertions.assertThatThrownBy(() -> spyOps.doCommit(metadataV2, metadataV1))
+    assertThatThrownBy(() -> spyOps.doCommit(metadataV2, metadataV1))
         .isInstanceOf(RuntimeException.class)
         .hasMessage(
             "org.apache.iceberg.hive.LockException: Metastore operation failed for hivedb.tbl");
   }
 
+  @Test
+  public void testPassThroughThriftExceptionsForHiveVersion_1()
+      throws TException, InterruptedException {
+    try (MockedStatic<HiveVersion> ignore = mockStatic(HiveVersion.class)) {
+      // default order is 0, meets the requirements of this test
+      HiveVersion version = mock(HiveVersion.class);
+      when(HiveVersion.current()).thenReturn(version);
+
+      doReturn(emptyLocks).when(spyClient).showLocks(any());
+      doThrow(new TException("Failed to connect to HMS"))
+          .doReturn(waitLockResponse)
+          .when(spyClient)
+          .lock(any());
+      doReturn(waitLockResponse)
+          .doReturn(acquiredLockResponse)
+          .when(spyClient)
+          .checkLock(eq(dummyLockId));
+      doNothing().when(spyClient).heartbeat(eq(0L), eq(dummyLockId));
+
+      assertThatThrownBy(() -> spyOps.doCommit(metadataV2, metadataV1))
+          .isInstanceOf(CommitFailedException.class)
+          .hasMessage(
+              "org.apache.iceberg.hive.LockException: Failed to find lock for table hivedb.tbl");
+    }
+  }
+
   @Test
   public void testPassThroughInterruptions() throws TException {
     doReturn(waitLockResponse).when(spyClient).lock(any());
@@ -427,7 +458,7 @@ public class TestHiveCommitLocks extends HiveTableBaseTest {
         .when(spyClient)
         .checkLock(eq(dummyLockId));
 
-    Assertions.assertThatThrownBy(() -> spyOps.doCommit(metadataV2, metadataV1))
+    assertThatThrownBy(() -> spyOps.doCommit(metadataV2, metadataV1))
         .isInstanceOf(CommitFailedException.class)
         .hasMessage(
             "org.apache.iceberg.hive.LockException: "
@@ -488,7 +519,7 @@ public class TestHiveCommitLocks extends HiveTableBaseTest {
         .when(spyClient)
         .heartbeat(eq(0L), eq(dummyLockId));
 
-    Assertions.assertThatThrownBy(() -> spyOps.doCommit(metadataV2, metadataV1))
+    assertThatThrownBy(() -> spyOps.doCommit(metadataV2, metadataV1))
         .isInstanceOf(CommitFailedException.class)
         .hasMessage(
             "org.apache.iceberg.hive.LockException: "
@@ -527,9 +558,10 @@ public class TestHiveCommitLocks extends HiveTableBaseTest {
 
     // Make sure that the expected parameter context values are set
     Map<String, String> context = contextCaptor.getValue().getProperties();
-    Assert.assertEquals(3, context.size());
-    Assert.assertEquals(
-        context.get("expected_parameter_key"), HiveTableOperations.METADATA_LOCATION_PROP);
-    Assert.assertEquals(context.get("expected_parameter_value"), metadataV2.metadataFileLocation());
+    assertThat(context).hasSize(3);
+    assertThat(HiveTableOperations.METADATA_LOCATION_PROP)
+        .isEqualTo(context.get("expected_parameter_key"));
+    assertThat(metadataV2.metadataFileLocation())
+        .isEqualTo(context.get("expected_parameter_value"));
   }
 }
