@@ -18,7 +18,12 @@
  */
 package org.apache.iceberg.avro;
 
+import static org.assertj.core.api.Assertions.assertThat;
+import static org.assertj.core.api.Assertions.assertThatThrownBy;
+import static org.assertj.core.api.Assertions.within;
+
 import java.io.IOException;
+import java.nio.file.Path;
 import java.util.List;
 import java.util.Map;
 import org.apache.avro.generic.GenericData.Record;
@@ -27,20 +32,15 @@ import org.apache.iceberg.TestHelpers;
 import org.apache.iceberg.relocated.com.google.common.collect.ImmutableList;
 import org.apache.iceberg.relocated.com.google.common.collect.ImmutableMap;
 import org.apache.iceberg.relocated.com.google.common.collect.Maps;
-import org.apache.iceberg.relocated.com.google.common.collect.Sets;
-import org.apache.iceberg.types.Comparators;
 import org.apache.iceberg.types.Types;
-import org.assertj.core.api.Assertions;
-import org.junit.Assert;
-import org.junit.Rule;
-import org.junit.Test;
-import org.junit.rules.TemporaryFolder;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.io.TempDir;
 
 public abstract class TestReadProjection {
   protected abstract Record writeAndRead(
       String desc, Schema writeSchema, Schema readSchema, Record record) throws IOException;
 
-  @Rule public TemporaryFolder temp = new TemporaryFolder();
+  @TempDir protected Path temp;
 
   @Test
   public void testFullProjection() throws Exception {
@@ -55,10 +55,11 @@ public abstract class TestReadProjection {
 
     Record projected = writeAndRead("full_projection", schema, schema, record);
 
-    Assert.assertEquals("Should contain the correct id value", 34L, (long) projected.get("id"));
+    assertThat((long) projected.get("id")).as("Should contain the correct id value").isEqualTo(34L);
 
-    int cmp = Comparators.charSequences().compare("test", (CharSequence) projected.get("data"));
-    Assert.assertTrue("Should contain the correct data value", cmp == 0);
+    assertThat(projected.get("data").toString())
+        .as("Should contain the correct data value")
+        .isEqualTo("test");
   }
 
   @Test
@@ -79,8 +80,10 @@ public abstract class TestReadProjection {
 
     Record projected = writeAndRead("full_projection", schema, reordered, record);
 
-    Assert.assertEquals("Should contain the correct 0 value", "test", projected.get(0).toString());
-    Assert.assertEquals("Should contain the correct 1 value", 34L, projected.get(1));
+    assertThat(projected.get(0).toString())
+        .as("Should contain the correct 0 value")
+        .isEqualTo("test");
+    assertThat(projected.get(1)).as("Should contain the correct 1 value").isEqualTo(34L);
   }
 
   @Test
@@ -102,9 +105,11 @@ public abstract class TestReadProjection {
 
     Record projected = writeAndRead("full_projection", schema, reordered, record);
 
-    Assert.assertNull("Should contain the correct 0 value", projected.get(0));
-    Assert.assertEquals("Should contain the correct 1 value", "test", projected.get(1).toString());
-    Assert.assertNull("Should contain the correct 2 value", projected.get(2));
+    assertThat(projected.get(0)).as("Should contain the correct 0 value").isNull();
+    assertThat(projected.get(1).toString())
+        .as("Should contain the correct 1 value")
+        .isEqualTo("test");
+    assertThat(projected.get(2)).as("Should contain the correct 2 value").isNull();
   }
 
   @Test
@@ -120,10 +125,9 @@ public abstract class TestReadProjection {
 
     Record projected = writeAndRead("empty_projection", schema, schema.select(), record);
 
-    Assert.assertNotNull("Should read a non-null record", projected);
+    assertThat(projected).as("Should read a non-null record").isNotNull();
     // this is expected because there are no values
-    Assertions.assertThatThrownBy(() -> projected.get(0))
-        .isInstanceOf(ArrayIndexOutOfBoundsException.class);
+    assertThatThrownBy(() -> projected.get(0)).isInstanceOf(ArrayIndexOutOfBoundsException.class);
   }
 
   @Test
@@ -141,15 +145,16 @@ public abstract class TestReadProjection {
 
     Record projected = writeAndRead("basic_projection_id", writeSchema, idOnly, record);
     TestHelpers.assertEmptyAvroField(projected, "data");
-    Assert.assertEquals("Should contain the correct id value", 34L, (long) projected.get("id"));
+    assertThat((long) projected.get("id")).as("Should contain the correct id value").isEqualTo(34L);
 
     Schema dataOnly = new Schema(Types.NestedField.optional(1, "data", Types.StringType.get()));
 
     projected = writeAndRead("basic_projection_data", writeSchema, dataOnly, record);
 
     TestHelpers.assertEmptyAvroField(projected, "id");
-    int cmp = Comparators.charSequences().compare("test", (CharSequence) projected.get("data"));
-    Assert.assertEquals("Should contain the correct data value", 0, cmp);
+    assertThat(projected.get("data").toString())
+        .as("Should contain the correct data value")
+        .isEqualTo("test");
   }
 
   @Test
@@ -170,9 +175,11 @@ public abstract class TestReadProjection {
 
     Record projected = writeAndRead("project_and_rename", writeSchema, readSchema, record);
 
-    Assert.assertEquals("Should contain the correct id value", 34L, (long) projected.get("id"));
-    int cmp = Comparators.charSequences().compare("test", (CharSequence) projected.get("renamed"));
-    Assert.assertTrue("Should contain the correct data/renamed value", cmp == 0);
+    assertThat((long) projected.get("id")).as("Should contain the correct id value").isEqualTo(34L);
+
+    assertThat(projected.get("renamed").toString())
+        .as("Should contain the correct data/renamed value")
+        .isEqualTo("test");
   }
 
   @Test
@@ -199,7 +206,7 @@ public abstract class TestReadProjection {
 
     Record projected = writeAndRead("id_only", writeSchema, idOnly, record);
     TestHelpers.assertEmptyAvroField(projected, "location");
-    Assert.assertEquals("Should contain the correct id value", 34L, (long) projected.get("id"));
+    assertThat((long) projected.get("id")).as("Should contain the correct id value").isEqualTo(34L);
 
     Schema latOnly =
         new Schema(
@@ -211,10 +218,11 @@ public abstract class TestReadProjection {
     projected = writeAndRead("latitude_only", writeSchema, latOnly, record);
     Record projectedLocation = (Record) projected.get("location");
     TestHelpers.assertEmptyAvroField(projected, "id");
-    Assert.assertNotNull("Should project location", projected.get("location"));
+    assertThat(projected.get("location")).as("Should project location").isNotNull();
     TestHelpers.assertEmptyAvroField(projectedLocation, "long");
-    Assert.assertEquals(
-        "Should project latitude", 52.995143f, (float) projectedLocation.get("lat"), 0.000001f);
+    assertThat((float) projectedLocation.get("lat"))
+        .as("Should project latitude")
+        .isCloseTo(52.995143f, within(0.000001f));
 
     Schema longOnly =
         new Schema(
@@ -226,20 +234,24 @@ public abstract class TestReadProjection {
     projected = writeAndRead("longitude_only", writeSchema, longOnly, record);
     projectedLocation = (Record) projected.get("location");
     TestHelpers.assertEmptyAvroField(projected, "id");
-    Assert.assertNotNull("Should project location", projected.get("location"));
+    assertThat(projected.get("location")).as("Should project location").isNotNull();
     TestHelpers.assertEmptyAvroField(projectedLocation, "lat");
-    Assert.assertEquals(
-        "Should project longitude", -1.539054f, (float) projectedLocation.get("long"), 0.000001f);
+    assertThat((float) projectedLocation.get("long"))
+        .as("Should project longitude")
+        .isCloseTo(-1.539054f, within(0.000001f));
 
     Schema locationOnly = writeSchema.select("location");
     projected = writeAndRead("location_only", writeSchema, locationOnly, record);
     projectedLocation = (Record) projected.get("location");
     TestHelpers.assertEmptyAvroField(projected, "id");
-    Assert.assertNotNull("Should project location", projected.get("location"));
-    Assert.assertEquals(
-        "Should project latitude", 52.995143f, (float) projectedLocation.get("lat"), 0.000001f);
-    Assert.assertEquals(
-        "Should project longitude", -1.539054f, (float) projectedLocation.get("long"), 0.000001f);
+    assertThat(projected.get("location")).as("Should project location").isNotNull();
+    assertThat((float) projectedLocation.get("lat"))
+        .as("Should project latitude")
+        .isCloseTo(52.995143f, within(0.000001f));
+
+    assertThat((float) projectedLocation.get("long"))
+        .as("Should project longitude")
+        .isCloseTo(-1.539054f, within(0.000001f));
   }
 
   @Test
@@ -261,26 +273,29 @@ public abstract class TestReadProjection {
     Schema idOnly = new Schema(Types.NestedField.required(0, "id", Types.LongType.get()));
 
     Record projected = writeAndRead("id_only", writeSchema, idOnly, record);
-    Assert.assertEquals("Should contain the correct id value", 34L, (long) projected.get("id"));
+    assertThat((long) projected.get("id")).as("Should contain the correct id value").isEqualTo(34L);
     TestHelpers.assertEmptyAvroField(projected, "properties");
 
     Schema keyOnly = writeSchema.select("properties.key");
     projected = writeAndRead("key_only", writeSchema, keyOnly, record);
     TestHelpers.assertEmptyAvroField(projected, "id");
-    Assert.assertEquals(
-        "Should project entire map", properties, toStringMap((Map) projected.get("properties")));
+    assertThat(toStringMap((Map) projected.get("properties")))
+        .as("Should project entire map")
+        .isEqualTo(properties);
 
     Schema valueOnly = writeSchema.select("properties.value");
     projected = writeAndRead("value_only", writeSchema, valueOnly, record);
     TestHelpers.assertEmptyAvroField(projected, "id");
-    Assert.assertEquals(
-        "Should project entire map", properties, toStringMap((Map) projected.get("properties")));
+    assertThat(toStringMap((Map) projected.get("properties")))
+        .as("Should project entire map")
+        .isEqualTo(properties);
 
     Schema mapOnly = writeSchema.select("properties");
     projected = writeAndRead("map_only", writeSchema, mapOnly, record);
     TestHelpers.assertEmptyAvroField(projected, "id");
-    Assert.assertEquals(
-        "Should project entire map", properties, toStringMap((Map) projected.get("properties")));
+    assertThat(toStringMap((Map) projected.get("properties")))
+        .as("Should project entire map")
+        .isEqualTo(properties);
   }
 
   private Map<String, ?> toStringMap(Map<?, ?> map) {
@@ -328,50 +343,54 @@ public abstract class TestReadProjection {
     Schema idOnly = new Schema(Types.NestedField.required(0, "id", Types.LongType.get()));
 
     Record projected = writeAndRead("id_only", writeSchema, idOnly, record);
-    Assert.assertEquals("Should contain the correct id value", 34L, (long) projected.get("id"));
+    assertThat((long) projected.get("id")).as("Should contain the correct id value").isEqualTo(34L);
     TestHelpers.assertEmptyAvroField(projected, "locations");
 
     projected = writeAndRead("all_locations", writeSchema, writeSchema.select("locations"), record);
     TestHelpers.assertEmptyAvroField(projected, "id");
-    Assert.assertEquals(
-        "Should project locations map",
-        record.get("locations"),
-        toStringMap((Map) projected.get("locations")));
+    assertThat(toStringMap((Map) projected.get("locations")))
+        .as("Should project locations map")
+        .isEqualTo(record.get("locations"));
 
     projected = writeAndRead("lat_only", writeSchema, writeSchema.select("locations.lat"), record);
     TestHelpers.assertEmptyAvroField(projected, "id");
     Map<String, ?> locations = toStringMap((Map) projected.get("locations"));
-    Assert.assertNotNull("Should project locations map", locations);
-    Assert.assertEquals(
-        "Should contain L1 and L2", Sets.newHashSet("L1", "L2"), locations.keySet());
+    assertThat(locations).as("Should project locations map").isNotNull();
+    assertThat(locations.keySet()).as("Should contain L1 and L2").containsExactly("L1", "L2");
     Record projectedL1 = (Record) locations.get("L1");
-    Assert.assertNotNull("L1 should not be null", projectedL1);
-    Assert.assertEquals(
-        "L1 should contain lat", 53.992811f, (float) projectedL1.get("lat"), 0.000001);
+    assertThat(projectedL1).as("L1 should not be null").isNotNull();
+    assertThat((float) projectedL1.get("lat"))
+        .as("L1 should contain lat")
+        .isCloseTo(53.992811f, within(0.000001f));
+
     TestHelpers.assertEmptyAvroField(projectedL1, "long");
     Record projectedL2 = (Record) locations.get("L2");
-    Assert.assertNotNull("L2 should not be null", projectedL2);
-    Assert.assertEquals(
-        "L2 should contain lat", 52.995143f, (float) projectedL2.get("lat"), 0.000001);
+    assertThat(projectedL2).as("L2 should not be null").isNotNull();
+    assertThat((float) projectedL2.get("lat"))
+        .as("L2 should contain lat")
+        .isCloseTo(52.995143f, within(0.000001f));
+
     TestHelpers.assertEmptyAvroField(projectedL2, "long");
 
     projected =
         writeAndRead("long_only", writeSchema, writeSchema.select("locations.long"), record);
     TestHelpers.assertEmptyAvroField(projected, "id");
     locations = toStringMap((Map) projected.get("locations"));
-    Assert.assertNotNull("Should project locations map", locations);
-    Assert.assertEquals(
-        "Should contain L1 and L2", Sets.newHashSet("L1", "L2"), locations.keySet());
+    assertThat(locations).as("Should project locations map").isNotNull();
+    assertThat(locations.keySet()).as("Should contain L1 and L2").containsExactly("L1", "L2");
     projectedL1 = (Record) locations.get("L1");
-    Assert.assertNotNull("L1 should not be null", projectedL1);
+    assertThat(projectedL1).as("L1 should not be null").isNotNull();
     TestHelpers.assertEmptyAvroField(projectedL1, "lat");
-    Assert.assertEquals(
-        "L1 should contain long", -1.542616f, (float) projectedL1.get("long"), 0.000001);
+    assertThat((float) projectedL1.get("long"))
+        .as("L1 should contain long")
+        .isCloseTo(-1.542616f, within(0.000001f));
+
     projectedL2 = (Record) locations.get("L2");
-    Assert.assertNotNull("L2 should not be null", projectedL2);
+    assertThat(projectedL2).as("L2 should not be null").isNotNull();
     TestHelpers.assertEmptyAvroField(projectedL2, "lat");
-    Assert.assertEquals(
-        "L2 should contain long", -1.539054f, (float) projectedL2.get("long"), 0.000001);
+    assertThat((float) projectedL2.get("long"))
+        .as("L2 should contain long")
+        .isCloseTo(-1.539054f, within(0.000001f));
 
     Schema latitiudeRenamed =
         new Schema(
@@ -388,19 +407,21 @@ public abstract class TestReadProjection {
     projected = writeAndRead("latitude_renamed", writeSchema, latitiudeRenamed, record);
     TestHelpers.assertEmptyAvroField(projected, "id");
     locations = toStringMap((Map) projected.get("locations"));
-    Assert.assertNotNull("Should project locations map", locations);
-    Assert.assertEquals(
-        "Should contain L1 and L2", Sets.newHashSet("L1", "L2"), locations.keySet());
+    assertThat(locations).as("Should project locations map").isNotNull();
+    assertThat(locations.keySet()).as("Should contain L1 and L2").containsExactly("L1", "L2");
     projectedL1 = (Record) locations.get("L1");
-    Assert.assertNotNull("L1 should not be null", projectedL1);
-    Assert.assertEquals(
-        "L1 should contain latitude", 53.992811f, (float) projectedL1.get("latitude"), 0.000001);
+    assertThat(projectedL1).as("L1 should not be null").isNotNull();
+    assertThat((float) projectedL1.get("latitude"))
+        .as("L1 should contain latitude")
+        .isCloseTo(53.992811f, within(0.000001f));
     TestHelpers.assertEmptyAvroField(projectedL1, "lat");
     TestHelpers.assertEmptyAvroField(projectedL1, "long");
     projectedL2 = (Record) locations.get("L2");
-    Assert.assertNotNull("L2 should not be null", projectedL2);
-    Assert.assertEquals(
-        "L2 should contain latitude", 52.995143f, (float) projectedL2.get("latitude"), 0.000001);
+    assertThat(projectedL2).as("L2 should not be null").isNotNull();
+    assertThat((float) projectedL2.get("latitude"))
+        .as("L2 should contain latitude")
+        .isCloseTo(52.995143f, within(0.000001f));
+
     TestHelpers.assertEmptyAvroField(projectedL2, "lat");
     TestHelpers.assertEmptyAvroField(projectedL2, "long");
   }
@@ -422,18 +443,18 @@ public abstract class TestReadProjection {
     Schema idOnly = new Schema(Types.NestedField.required(0, "id", Types.LongType.get()));
 
     Record projected = writeAndRead("id_only", writeSchema, idOnly, record);
-    Assert.assertEquals("Should contain the correct id value", 34L, (long) projected.get("id"));
+    assertThat((long) projected.get("id")).as("Should contain the correct id value").isEqualTo(34L);
     TestHelpers.assertEmptyAvroField(projected, "values");
 
     Schema elementOnly = writeSchema.select("values.element");
     projected = writeAndRead("element_only", writeSchema, elementOnly, record);
     TestHelpers.assertEmptyAvroField(projected, "id");
-    Assert.assertEquals("Should project entire list", values, projected.get("values"));
+    assertThat(projected.get("values")).as("Should project entire list").isEqualTo(values);
 
     Schema listOnly = writeSchema.select("values");
     projected = writeAndRead("list_only", writeSchema, listOnly, record);
     TestHelpers.assertEmptyAvroField(projected, "id");
-    Assert.assertEquals("Should project entire list", values, projected.get("values"));
+    assertThat(projected.get("values")).as("Should project entire list").isEqualTo(values);
   }
 
   @Test
@@ -468,37 +489,38 @@ public abstract class TestReadProjection {
     Schema idOnly = new Schema(Types.NestedField.required(0, "id", Types.LongType.get()));
 
     Record projected = writeAndRead("id_only", writeSchema, idOnly, record);
-    Assert.assertEquals("Should contain the correct id value", 34L, (long) projected.get("id"));
+    assertThat((long) projected.get("id")).as("Should contain the correct id value").isEqualTo(34L);
     TestHelpers.assertEmptyAvroField(projected, "points");
 
     projected = writeAndRead("all_points", writeSchema, writeSchema.select("points"), record);
     TestHelpers.assertEmptyAvroField(projected, "id");
-    Assert.assertEquals(
-        "Should project points list", record.get("points"), projected.get("points"));
+    assertThat(projected.get("points"))
+        .as("Should project points list")
+        .isEqualTo(record.get("points"));
 
     projected = writeAndRead("x_only", writeSchema, writeSchema.select("points.x"), record);
     TestHelpers.assertEmptyAvroField(projected, "id");
-    Assert.assertNotNull("Should project points list", projected.get("points"));
+    assertThat(projected.get("points")).as("Should project points list").isNotNull();
     List<Record> points = (List<Record>) projected.get("points");
-    Assert.assertEquals("Should read 2 points", 2, points.size());
+    assertThat(points).as("Should read 2 points").hasSize(2);
     Record projectedP1 = points.get(0);
-    Assert.assertEquals("Should project x", 1, (int) projectedP1.get("x"));
+    assertThat((int) projectedP1.get("x")).as("Should project x").isEqualTo(1);
     TestHelpers.assertEmptyAvroField(projectedP1, "y");
     Record projectedP2 = points.get(1);
-    Assert.assertEquals("Should project x", 3, (int) projectedP2.get("x"));
+    assertThat((int) projectedP2.get("x")).as("Should project x").isEqualTo(3);
     TestHelpers.assertEmptyAvroField(projectedP2, "y");
 
     projected = writeAndRead("y_only", writeSchema, writeSchema.select("points.y"), record);
     TestHelpers.assertEmptyAvroField(projected, "id");
-    Assert.assertNotNull("Should project points list", projected.get("points"));
+    assertThat(projected.get("points")).as("Should project points list").isNotNull();
     points = (List<Record>) projected.get("points");
-    Assert.assertEquals("Should read 2 points", 2, points.size());
+    assertThat(points).as("Should read 2 points").hasSize(2);
     projectedP1 = points.get(0);
     TestHelpers.assertEmptyAvroField(projectedP1, "x");
-    Assert.assertEquals("Should project y", 2, (int) projectedP1.get("y"));
+    assertThat((int) projectedP1.get("y")).as("Should project y").isEqualTo(2);
     projectedP2 = points.get(1);
     TestHelpers.assertEmptyAvroField(projectedP2, "x");
-    Assert.assertNull("Should project null y", projectedP2.get("y"));
+    assertThat(projectedP2.get("y")).as("Should project null y").isNull();
 
     Schema yRenamed =
         new Schema(
@@ -512,16 +534,16 @@ public abstract class TestReadProjection {
 
     projected = writeAndRead("y_renamed", writeSchema, yRenamed, record);
     TestHelpers.assertEmptyAvroField(projected, "id");
-    Assert.assertNotNull("Should project points list", projected.get("points"));
+    assertThat(projected.get("points")).as("Should project points list").isNotNull();
     points = (List<Record>) projected.get("points");
-    Assert.assertEquals("Should read 2 points", 2, points.size());
+    assertThat(points).as("Should read 2 points").hasSize(2);
     projectedP1 = points.get(0);
     TestHelpers.assertEmptyAvroField(projectedP1, "x");
     TestHelpers.assertEmptyAvroField(projectedP1, "y");
-    Assert.assertEquals("Should project z", 2, (int) projectedP1.get("z"));
+    assertThat((int) projectedP1.get("z")).as("Should project z").isEqualTo(2);
     projectedP2 = points.get(1);
     TestHelpers.assertEmptyAvroField(projectedP2, "x");
     TestHelpers.assertEmptyAvroField(projectedP2, "y");
-    Assert.assertNull("Should project null z", projectedP2.get("z"));
+    assertThat(projectedP2.get("z")).as("Should project null z").isNull();
   }
 }
