@@ -20,18 +20,18 @@
 package org.apache.spark.sql.catalyst.analysis
 
 import org.apache.spark.sql.AnalysisException
+import org.apache.spark.sql.catalyst.ProjectingInternalRow
 import org.apache.spark.sql.catalyst.expressions.Alias
-import org.apache.spark.sql.catalyst.expressions.And
 import org.apache.spark.sql.catalyst.expressions.Attribute
 import org.apache.spark.sql.catalyst.expressions.AttributeReference
+import org.apache.spark.sql.catalyst.expressions.AttributeSet
 import org.apache.spark.sql.catalyst.expressions.Expression
+import org.apache.spark.sql.catalyst.expressions.ExtendedV2ExpressionUtils
 import org.apache.spark.sql.catalyst.expressions.IsNotNull
 import org.apache.spark.sql.catalyst.expressions.Literal
 import org.apache.spark.sql.catalyst.expressions.Literal.FalseLiteral
 import org.apache.spark.sql.catalyst.expressions.Literal.TrueLiteral
 import org.apache.spark.sql.catalyst.expressions.MonotonicallyIncreasingID
-import org.apache.spark.sql.catalyst.expressions.PredicateHelper
-import org.apache.spark.sql.catalyst.expressions.V2ExpressionUtils
 import org.apache.spark.sql.catalyst.plans.FullOuter
 import org.apache.spark.sql.catalyst.plans.Inner
 import org.apache.spark.sql.catalyst.plans.LeftAnti
@@ -51,20 +51,21 @@ import org.apache.spark.sql.catalyst.plans.logical.MergeRows
 import org.apache.spark.sql.catalyst.plans.logical.NO_BROADCAST_HASH
 import org.apache.spark.sql.catalyst.plans.logical.NoStatsUnaryNode
 import org.apache.spark.sql.catalyst.plans.logical.Project
-import org.apache.spark.sql.catalyst.plans.logical.ReplaceIcebergData
+import org.apache.spark.sql.catalyst.plans.logical.ReplaceData
 import org.apache.spark.sql.catalyst.plans.logical.UpdateAction
-import org.apache.spark.sql.catalyst.plans.logical.WriteIcebergDelta
+import org.apache.spark.sql.catalyst.plans.logical.WriteDelta
 import org.apache.spark.sql.catalyst.util.RowDeltaUtils._
 import org.apache.spark.sql.catalyst.util.WriteDeltaProjections
-import org.apache.spark.sql.connector.catalog.SupportsRowLevelOperations
 import org.apache.spark.sql.connector.expressions.FieldReference
 import org.apache.spark.sql.connector.expressions.NamedReference
-import org.apache.spark.sql.connector.write.RowLevelOperation.Command.MERGE
+import org.apache.spark.sql.connector.iceberg.catalog.SupportsRowLevelOperations
+import org.apache.spark.sql.connector.iceberg.write.RowLevelOperation.Command.MERGE
+import org.apache.spark.sql.connector.iceberg.write.SupportsDelta
 import org.apache.spark.sql.connector.write.RowLevelOperationTable
-import org.apache.spark.sql.connector.write.SupportsDelta
 import org.apache.spark.sql.execution.datasources.v2.DataSourceV2Relation
 import org.apache.spark.sql.types.IntegerType
-import org.apache.spark.sql.util.CaseInsensitiveStringMap
+import org.apache.spark.sql.types.StructField
+import org.apache.spark.sql.types.StructType
 
 /**
  * Assigns a rewrite plan for v2 tables that support rewriting data to handle MERGE statements.
@@ -72,7 +73,7 @@ import org.apache.spark.sql.util.CaseInsensitiveStringMap
  * This rule assumes the commands have been fully resolved and all assignments have been aligned.
  * That's why it must be run after AlignRowLevelCommandAssignments.
  */
-object RewriteMergeIntoTable extends RewriteRowLevelIcebergCommand with PredicateHelper {
+object RewriteMergeIntoTable extends RewriteRowLevelCommand {
 
   private final val ROW_FROM_SOURCE = "__row_from_source"
   private final val ROW_FROM_TARGET = "__row_from_target"
@@ -80,6 +81,7 @@ object RewriteMergeIntoTable extends RewriteRowLevelIcebergCommand with Predicat
 
   private final val ROW_FROM_SOURCE_REF = FieldReference(ROW_FROM_SOURCE)
   private final val ROW_FROM_TARGET_REF = FieldReference(ROW_FROM_TARGET)
+  private final val ROW_ID_REF = FieldReference(ROW_ID)
 
   override def apply(plan: LogicalPlan): LogicalPlan = plan resolveOperators {
     case m @ MergeIntoIcebergTable(aliasedTable, source, cond, matchedActions, notMatchedActions, None)
@@ -123,7 +125,7 @@ object RewriteMergeIntoTable extends RewriteRowLevelIcebergCommand with Predicat
           val joinPlan = Join(source, r, LeftAnti, Some(cond), JoinHint.NONE)
 
           val notMatchedConditions = notMatchedActions.map(actionCondition)
-          val notMatchedOutputs = notMatchedActions.map(notMatchedActionOutput(_, Nil))
+          val notMatchedOutputs = notMatchedActions.map(actionOutput(_, Nil))
 
           // merge rows as there are multiple not matched actions
           val mergeRows = MergeRows(
@@ -134,6 +136,7 @@ object RewriteMergeIntoTable extends RewriteRowLevelIcebergCommand with Predicat
             notMatchedConditions = notMatchedConditions,
             notMatchedOutputs = notMatchedOutputs,
             targetOutput = Nil,
+            rowIdAttrs = Nil,
             performCardinalityCheck = false,
             emitNotMatchedTargetRows = false,
             output = buildMergeRowsOutput(Nil, notMatchedOutputs, r.output),
@@ -150,8 +153,9 @@ object RewriteMergeIntoTable extends RewriteRowLevelIcebergCommand with Predicat
 
       EliminateSubqueryAliases(aliasedTable) match {
         case r @ DataSourceV2Relation(tbl: SupportsRowLevelOperations, _, _, _, _) =>
-          val table = buildOperationTable(tbl, MERGE, CaseInsensitiveStringMap.empty())
-          val rewritePlan = table.operation match {
+          val operation = buildRowLevelOperation(tbl, MERGE)
+          val table = RowLevelOperationTable(tbl, operation)
+          val rewritePlan = operation match {
             case _: SupportsDelta =>
               buildWriteDeltaPlan(r, table, source, cond, matchedActions, notMatchedActions)
             case _ =>
@@ -172,26 +176,20 @@ object RewriteMergeIntoTable extends RewriteRowLevelIcebergCommand with Predicat
       source: LogicalPlan,
       cond: Expression,
       matchedActions: Seq[MergeAction],
-      notMatchedActions: Seq[MergeAction]): ReplaceIcebergData = {
+      notMatchedActions: Seq[MergeAction]): ReplaceData = {
 
     // resolve all needed attrs (e.g. metadata attrs for grouping data on write)
     val metadataAttrs = resolveRequiredMetadataAttrs(relation, operationTable.operation)
 
     // construct a scan relation and include all required metadata columns
-    val readRelation = buildRelationWithAttrs(relation, operationTable, metadataAttrs)
+    val readRelation = buildReadRelation(relation, operationTable, metadataAttrs)
     val readAttrs = readRelation.output
 
-    val performCardinalityCheck = isCardinalityCheckNeeded(matchedActions)
-
     // project an extra column to check if a target row exists after the join
-    // if needed, project a synthetic row ID to perform the cardinality check
+    // project a synthetic row ID to perform the cardinality check
     val rowFromTarget = Alias(TrueLiteral, ROW_FROM_TARGET)()
-    val targetTableProjExprs = if (performCardinalityCheck) {
-      val rowId = Alias(MonotonicallyIncreasingID(), ROW_ID)()
-      readAttrs ++ Seq(rowFromTarget, rowId)
-    } else {
-      readAttrs :+ rowFromTarget
-    }
+    val rowId = Alias(MonotonicallyIncreasingID(), ROW_ID)()
+    val targetTableProjExprs = readAttrs ++ Seq(rowFromTarget, rowId)
     val targetTableProj = Project(targetTableProjExprs, readRelation)
 
     // project an extra column to check if a source row exists after the join
@@ -206,12 +204,15 @@ object RewriteMergeIntoTable extends RewriteRowLevelIcebergCommand with Predicat
     val joinHint = JoinHint(leftHint = Some(HintInfo(Some(NO_BROADCAST_HASH))), rightHint = None)
     val joinPlan = Join(NoStatsUnaryNode(targetTableProj), sourceTableProj, joinType, Some(cond), joinHint)
 
-    val matchedConditions = matchedActions.map(actionCondition)
-    val matchedOutputs = matchedActions.map(matchedActionOutput(_, metadataAttrs))
+    // add an extra matched action to output the original row if none of the actual actions matched
+    // this is needed to keep target rows that should be copied over
+    val matchedConditions = matchedActions.map(actionCondition) :+ TrueLiteral
+    val matchedOutputs = matchedActions.map(actionOutput(_, metadataAttrs)) :+ readAttrs
 
     val notMatchedConditions = notMatchedActions.map(actionCondition)
-    val notMatchedOutputs = notMatchedActions.map(notMatchedActionOutput(_, metadataAttrs))
+    val notMatchedOutputs = notMatchedActions.map(actionOutput(_, metadataAttrs))
 
+    val rowIdAttr = resolveAttrRef(ROW_ID_REF, joinPlan)
     val rowFromSourceAttr = resolveAttrRef(ROW_FROM_SOURCE_REF, joinPlan)
     val rowFromTargetAttr = resolveAttrRef(ROW_FROM_TARGET_REF, joinPlan)
 
@@ -223,14 +224,15 @@ object RewriteMergeIntoTable extends RewriteRowLevelIcebergCommand with Predicat
       notMatchedConditions = notMatchedConditions,
       notMatchedOutputs = notMatchedOutputs,
       targetOutput = readAttrs,
-      performCardinalityCheck = performCardinalityCheck,
+      rowIdAttrs = Seq(rowIdAttr),
+      performCardinalityCheck = isCardinalityCheckNeeded(matchedActions),
       emitNotMatchedTargetRows = true,
       output = buildMergeRowsOutput(matchedOutputs, notMatchedOutputs, readAttrs),
       joinPlan)
 
     // build a plan to replace read groups in the table
     val writeRelation = relation.copy(table = operationTable)
-    ReplaceIcebergData(writeRelation, mergeRows, relation)
+    ReplaceData(writeRelation, mergeRows, relation)
   }
 
   // build a rewrite plan for sources that support row deltas
@@ -240,7 +242,7 @@ object RewriteMergeIntoTable extends RewriteRowLevelIcebergCommand with Predicat
       source: LogicalPlan,
       cond: Expression,
       matchedActions: Seq[MergeAction],
-      notMatchedActions: Seq[MergeAction]): WriteIcebergDelta = {
+      notMatchedActions: Seq[MergeAction]): WriteDelta = {
 
     // resolve all needed attrs (e.g. row ID and any required metadata attrs)
     val rowAttrs = relation.output
@@ -248,23 +250,12 @@ object RewriteMergeIntoTable extends RewriteRowLevelIcebergCommand with Predicat
     val metadataAttrs = resolveRequiredMetadataAttrs(relation, operationTable.operation)
 
     // construct a scan relation and include all required metadata columns
-    val readRelation = buildRelationWithAttrs(relation, operationTable, rowIdAttrs ++ metadataAttrs)
+    val readRelation = buildReadRelation(relation, operationTable, metadataAttrs, rowIdAttrs)
     val readAttrs = readRelation.output
 
-    val (targetCond, joinCond) = splitMergeCond(cond, readRelation)
-
-    val performCardinalityCheck = isCardinalityCheckNeeded(matchedActions)
-
     // project an extra column to check if a target row exists after the join
-    // if needed, project a synthetic row ID to perform the cardinality check
-    val rowFromTarget = Alias(TrueLiteral, ROW_FROM_TARGET)()
-    val targetTableProjExprs = if (performCardinalityCheck) {
-      val rowId = Alias(MonotonicallyIncreasingID(), ROW_ID)()
-      readAttrs ++ Seq(rowFromTarget, rowId)
-    } else {
-      readAttrs :+ rowFromTarget
-    }
-    val targetTableProj = Project(targetTableProjExprs, Filter(targetCond, readRelation))
+    val targetTableProjExprs = readAttrs :+ Alias(TrueLiteral, ROW_FROM_TARGET)()
+    val targetTableProj = Project(targetTableProjExprs, readRelation)
 
     // project an extra column to check if a source row exists after the join
     val sourceTableProjExprs = source.output :+ Alias(TrueLiteral, ROW_FROM_SOURCE)()
@@ -275,19 +266,16 @@ object RewriteMergeIntoTable extends RewriteRowLevelIcebergCommand with Predicat
     // also disable broadcasts for the target table to perform the cardinality check
     val joinType = if (notMatchedActions.isEmpty) Inner else RightOuter
     val joinHint = JoinHint(leftHint = Some(HintInfo(Some(NO_BROADCAST_HASH))), rightHint = None)
-    val joinPlan = Join(NoStatsUnaryNode(targetTableProj), sourceTableProj, joinType, Some(joinCond), joinHint)
+    val joinPlan = Join(NoStatsUnaryNode(targetTableProj), sourceTableProj, joinType, Some(cond), joinHint)
 
+    val deleteRowValues = buildDeltaDeleteRowValues(rowAttrs, rowIdAttrs)
     val metadataReadAttrs = readAttrs.filterNot(relation.outputSet.contains)
 
     val matchedConditions = matchedActions.map(actionCondition)
-    val matchedOutputs = matchedActions.map { action =>
-      matchedDeltaActionOutput(action, rowAttrs, rowIdAttrs, metadataReadAttrs)
-    }
+    val matchedOutputs = matchedActions.map(deltaActionOutput(_, deleteRowValues, metadataReadAttrs))
 
     val notMatchedConditions = notMatchedActions.map(actionCondition)
-    val notMatchedOutputs = notMatchedActions.map { action =>
-      notMatchedDeltaActionOutput(action, metadataReadAttrs)
-    }
+    val notMatchedOutputs = notMatchedActions.map(deltaActionOutput(_, deleteRowValues, metadataReadAttrs))
 
     val operationTypeAttr = AttributeReference(OPERATION_COLUMN, IntegerType, nullable = false)()
     val rowFromSourceAttr = resolveAttrRef(ROW_FROM_SOURCE_REF, joinPlan)
@@ -305,92 +293,79 @@ object RewriteMergeIntoTable extends RewriteRowLevelIcebergCommand with Predicat
       notMatchedOutputs = notMatchedOutputs,
       // only needed if emitting unmatched target rows
       targetOutput = Nil,
-      performCardinalityCheck = performCardinalityCheck,
+      rowIdAttrs = rowIdAttrs,
+      performCardinalityCheck = isCardinalityCheckNeeded(matchedActions),
       emitNotMatchedTargetRows = false,
       output = mergeRowsOutput,
       joinPlan)
 
     // build a plan to write the row delta to the table
     val writeRelation = relation.copy(table = operationTable)
-    val projections = buildDeltaProjections(mergeRows, rowAttrs, rowIdAttrs, metadataAttrs)
-    WriteIcebergDelta(writeRelation, mergeRows, relation, projections)
+    val projections = buildMergeDeltaProjections(mergeRows, rowAttrs, rowIdAttrs, metadataAttrs)
+    WriteDelta(writeRelation, mergeRows, relation, projections)
   }
 
   private def actionCondition(action: MergeAction): Expression = {
     action.condition.getOrElse(TrueLiteral)
   }
 
-  private def matchedActionOutput(
+  private def actionOutput(
       clause: MergeAction,
-      metadataAttrs: Seq[Attribute]): Seq[Seq[Expression]] = {
+      metadataAttrs: Seq[Attribute]): Seq[Expression] = {
 
     clause match {
       case u: UpdateAction =>
-        Seq(u.assignments.map(_.value) ++ metadataAttrs)
+        u.assignments.map(_.value) ++ metadataAttrs
 
       case _: DeleteAction =>
         Nil
 
-      case other =>
-        throw new AnalysisException(s"Unexpected WHEN MATCHED action: $other")
-    }
-  }
-
-  private def notMatchedActionOutput(
-      clause: MergeAction,
-      metadataAttrs: Seq[Attribute]): Seq[Expression] = {
-
-    clause match {
       case i: InsertAction =>
         i.assignments.map(_.value) ++ metadataAttrs.map(attr => Literal(null, attr.dataType))
 
       case other =>
-        throw new AnalysisException(s"Unexpected WHEN NOT MATCHED action: $other")
+        throw new AnalysisException(s"Unexpected action: $other")
     }
   }
 
-  private def matchedDeltaActionOutput(
-      action: MergeAction,
-      rowAttrs: Seq[Attribute],
-      rowIdAttrs: Seq[Attribute],
-      metadataAttrs: Seq[Attribute]): Seq[Seq[Expression]] = {
-
-    action match {
-      case u: UpdateAction =>
-        val delete = deltaDeleteOutput(rowAttrs, rowIdAttrs, metadataAttrs)
-        val insert = deltaInsertOutput(u.assignments.map(_.value), metadataAttrs)
-        Seq(delete, insert)
-
-      case _: DeleteAction =>
-        val delete = deltaDeleteOutput(rowAttrs, rowIdAttrs, metadataAttrs)
-        Seq(delete)
-
-      case other =>
-        throw new AnalysisException(s"Unexpected WHEN MATCHED action: $other")
-    }
-  }
-
-  private def notMatchedDeltaActionOutput(
+  private def deltaActionOutput(
       action: MergeAction,
+      deleteRowValues: Seq[Expression],
       metadataAttrs: Seq[Attribute]): Seq[Expression] = {
 
     action match {
+      case u: UpdateAction =>
+        Seq(Literal(UPDATE_OPERATION)) ++ u.assignments.map(_.value) ++ metadataAttrs
+
+      case _: DeleteAction =>
+        Seq(Literal(DELETE_OPERATION)) ++ deleteRowValues ++ metadataAttrs
+
       case i: InsertAction =>
-        deltaInsertOutput(i.assignments.map(_.value), metadataAttrs)
+        val metadataAttrValues = metadataAttrs.map(attr => Literal(null, attr.dataType))
+        Seq(Literal(INSERT_OPERATION)) ++ i.assignments.map(_.value) ++ metadataAttrValues
 
       case other =>
-        throw new AnalysisException(s"Unexpected WHEN NOT MATCHED action: $other")
+        throw new AnalysisException(s"Unexpected action: $other")
     }
   }
 
   private def buildMergeRowsOutput(
-      matchedOutputs: Seq[Seq[Seq[Expression]]],
+      matchedOutputs: Seq[Seq[Expression]],
       notMatchedOutputs: Seq[Seq[Expression]],
       attrs: Seq[Attribute]): Seq[Attribute] = {
 
-    // collect all outputs from matched and not matched actions (ignoring actions that discard rows)
-    val outputs = matchedOutputs.flatten.filter(_.nonEmpty) ++ notMatchedOutputs.filter(_.nonEmpty)
-    buildMergingOutput(outputs, attrs)
+    // collect all outputs from matched and not matched actions (ignoring DELETEs)
+    val outputs = matchedOutputs.filter(_.nonEmpty) ++ notMatchedOutputs.filter(_.nonEmpty)
+
+    // build a correct nullability map for output attributes
+    // an attribute is nullable if at least one matched or not matched action may produce null
+    val nullabilityMap = attrs.indices.map { index =>
+      index -> outputs.exists(output => output(index).nullable)
+    }.toMap
+
+    attrs.zipWithIndex.map { case (attr, index) =>
+      AttributeReference(attr.name, attr.dataType, nullabilityMap(index), attr.metadata)()
+    }
   }
 
   private def isCardinalityCheckNeeded(actions: Seq[MergeAction]): Boolean = actions match {
@@ -398,30 +373,70 @@ object RewriteMergeIntoTable extends RewriteRowLevelIcebergCommand with Predicat
     case _ => true
   }
 
-  private def resolveAttrRef(ref: NamedReference, plan: LogicalPlan): AttributeReference = {
-    V2ExpressionUtils.resolveRef[AttributeReference](ref, plan)
+  private def buildDeltaDeleteRowValues(
+      rowAttrs: Seq[Attribute],
+      rowIdAttrs: Seq[Attribute]): Seq[Expression] = {
+
+    // nullify all row attrs that are not part of the row ID
+    val rowIdAttSet = AttributeSet(rowIdAttrs)
+    rowAttrs.map {
+      case attr if rowIdAttSet.contains(attr) => attr
+      case attr => Literal(null, attr.dataType)
+    }
   }
 
-  private def buildDeltaProjections(
+  private def resolveAttrRef(ref: NamedReference, plan: LogicalPlan): AttributeReference = {
+    ExtendedV2ExpressionUtils.resolveRef[AttributeReference](ref, plan)
+  }
+
+  private def buildMergeDeltaProjections(
       mergeRows: MergeRows,
       rowAttrs: Seq[Attribute],
       rowIdAttrs: Seq[Attribute],
       metadataAttrs: Seq[Attribute]): WriteDeltaProjections = {
 
-    val outputs = mergeRows.matchedOutputs.flatten ++ mergeRows.notMatchedOutputs
-    buildDeltaProjections(mergeRows, outputs, rowAttrs, rowIdAttrs, metadataAttrs)
+    val outputAttrs = mergeRows.output
+
+    val outputs = mergeRows.matchedOutputs ++ mergeRows.notMatchedOutputs
+    val insertAndUpdateOutputs = outputs.filterNot(_.head == Literal(DELETE_OPERATION))
+    val updateAndDeleteOutputs = outputs.filterNot(_.head == Literal(INSERT_OPERATION))
+
+    val rowProjection = if (rowAttrs.nonEmpty) {
+      Some(newLazyProjection(insertAndUpdateOutputs, outputAttrs, rowAttrs))
+    } else {
+      None
+    }
+
+    val rowIdProjection = newLazyProjection(updateAndDeleteOutputs, outputAttrs, rowIdAttrs)
+
+    val metadataProjection = if (metadataAttrs.nonEmpty) {
+      Some(newLazyProjection(updateAndDeleteOutputs, outputAttrs, metadataAttrs))
+    } else {
+      None
+    }
+
+    WriteDeltaProjections(rowProjection, rowIdProjection, metadataProjection)
   }
 
-  // splits the MERGE condition into a predicate that references columns only from the target table,
-  // which can be pushed down, and a predicate used as a join condition to find matches
-  private def splitMergeCond(
-      cond: Expression,
-      targetTable: LogicalPlan): (Expression, Expression) = {
+  // the projection is done by name, ignoring expr IDs
+  private def newLazyProjection(
+      outputs: Seq[Seq[Expression]],
+      outputAttrs: Seq[Attribute],
+      projectedAttrs: Seq[Attribute]): ProjectingInternalRow = {
 
-    val (targetPredicates, joinPredicates) = splitConjunctivePredicates(cond)
-      .partition(_.references.subsetOf(targetTable.outputSet))
-    val targetCond = targetPredicates.reduceOption(And).getOrElse(TrueLiteral)
-    val joinCond = joinPredicates.reduceOption(And).getOrElse(TrueLiteral)
-    (targetCond, joinCond)
+    val projectedOrdinals = projectedAttrs.map(attr => outputAttrs.indexWhere(_.name == attr.name))
+
+    val structFields = projectedAttrs.zip(projectedOrdinals).map { case (attr, ordinal) =>
+      // output attr is nullable if at least one action may produce null for that attr
+      // but row ID and metadata attrs are projected only in update/delete actions and
+      // row attrs are projected only in insert/update actions
+      // that's why the projection schema must rely only on relevant action outputs
+      // instead of blindly inheriting the output attr nullability
+      val nullable = outputs.exists(output => output(ordinal).nullable)
+      StructField(attr.name, attr.dataType, nullable, attr.metadata)
+    }
+    val schema = StructType(structFields)
+
+    ProjectingInternalRow(schema, projectedOrdinals)
   }
 }
