@@ -32,19 +32,14 @@ import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
 import org.apache.iceberg.DataFile;
 import org.apache.iceberg.RowLevelOperationMode;
-import org.apache.iceberg.Snapshot;
 import org.apache.iceberg.Table;
 import org.apache.iceberg.TableProperties;
 import org.apache.iceberg.data.GenericRecord;
 import org.apache.iceberg.relocated.com.google.common.collect.ImmutableList;
 import org.apache.iceberg.relocated.com.google.common.collect.ImmutableMap;
-import org.apache.iceberg.relocated.com.google.common.collect.Iterables;
 import org.apache.iceberg.relocated.com.google.common.util.concurrent.MoreExecutors;
 import org.apache.iceberg.spark.Spark3Util;
-import org.apache.iceberg.spark.SparkSQLProperties;
-import org.apache.iceberg.util.SnapshotUtil;
 import org.apache.spark.sql.Encoders;
-import org.apache.spark.sql.internal.SQLConf;
 import org.assertj.core.api.Assertions;
 import org.junit.Assert;
 import org.junit.Assume;
@@ -58,9 +53,8 @@ public class TestCopyOnWriteMerge extends TestMerge {
       Map<String, String> config,
       String fileFormat,
       boolean vectorized,
-      String distributionMode,
-      String branch) {
-    super(catalogName, implementation, config, fileFormat, vectorized, distributionMode, branch);
+      String distributionMode) {
+    super(catalogName, implementation, config, fileFormat, vectorized, distributionMode);
   }
 
   @Override
@@ -83,7 +77,6 @@ public class TestCopyOnWriteMerge extends TestMerge {
         tableName, MERGE_ISOLATION_LEVEL, "snapshot");
 
     sql("INSERT INTO TABLE %s VALUES (1, 'hr')", tableName);
-    createBranchIfNeeded();
 
     Table table = Spark3Util.loadIcebergTable(spark, tableName);
 
@@ -155,46 +148,4 @@ public class TestCopyOnWriteMerge extends TestMerge {
     executorService.shutdown();
     Assert.assertTrue("Timeout", executorService.awaitTermination(2, TimeUnit.MINUTES));
   }
-
-  @Test
-  public void testRuntimeFilteringWithReportedPartitioning() {
-    createAndInitTable("id INT, dep STRING");
-    sql("ALTER TABLE %s ADD PARTITION FIELD dep", tableName);
-
-    append(tableName, "{ \"id\": 1, \"dep\": \"hr\" }\n" + "{ \"id\": 3, \"dep\": \"hr\" }");
-    createBranchIfNeeded();
-    append(
-        commitTarget(),
-        "{ \"id\": 1, \"dep\": \"hardware\" }\n" + "{ \"id\": 2, \"dep\": \"hardware\" }");
-
-    createOrReplaceView("source", Collections.singletonList(2), Encoders.INT());
-
-    Map<String, String> sqlConf =
-        ImmutableMap.of(
-            SQLConf.V2_BUCKETING_ENABLED().key(),
-            "true",
-            SparkSQLProperties.PRESERVE_DATA_GROUPING,
-            "true");
-
-    withSQLConf(
-        sqlConf,
-        () ->
-            sql(
-                "MERGE INTO %s t USING source s "
-                    + "ON t.id == s.value "
-                    + "WHEN MATCHED THEN "
-                    + "  UPDATE SET id = -1",
-                commitTarget()));
-
-    Table table = validationCatalog.loadTable(tableIdent);
-    Assert.assertEquals("Should have 3 snapshots", 3, Iterables.size(table.snapshots()));
-
-    Snapshot currentSnapshot = SnapshotUtil.latestSnapshot(table, branch);
-    validateCopyOnWrite(currentSnapshot, "1", "1", "1");
-
-    assertEquals(
-        "Should have expected rows",
-        ImmutableList.of(row(-1, "hardware"), row(1, "hardware"), row(1, "hr"), row(3, "hr")),
-        sql("SELECT * FROM %s ORDER BY id, dep", selectTarget()));
-  }
 }
