@@ -69,7 +69,7 @@ public class TestConflictValidation extends SparkExtensionsTestBase {
 
     // Validating from previous snapshot finds conflicts
     Dataset<Row> conflictingDf = spark.createDataFrame(records, SimpleRecord.class);
-    AssertHelpers.assertThrows(
+    AssertHelpers.assertThrowsCause(
         "Conflicting new data files should throw exception",
         ValidationException.class,
         "Found conflicting files that can contain records matching ref(name=\"id\") == 1:",
@@ -111,7 +111,7 @@ public class TestConflictValidation extends SparkExtensionsTestBase {
     // Validating from previous snapshot finds conflicts
     List<SimpleRecord> conflictingRecords = Lists.newArrayList(new SimpleRecord(1, "a"));
     Dataset<Row> conflictingDf = spark.createDataFrame(conflictingRecords, SimpleRecord.class);
-    AssertHelpers.assertThrows(
+    AssertHelpers.assertThrowsCause(
         "Conflicting new delete files should throw exception",
         ValidationException.class,
         "Found new conflicting delete files that can apply to records matching ref(name=\"id\") == 1:",
@@ -149,7 +149,7 @@ public class TestConflictValidation extends SparkExtensionsTestBase {
     // Validating from previous snapshot finds conflicts
     List<SimpleRecord> conflictingRecords = Lists.newArrayList(new SimpleRecord(1, "a"));
     Dataset<Row> conflictingDf = spark.createDataFrame(conflictingRecords, SimpleRecord.class);
-    AssertHelpers.assertThrows(
+    AssertHelpers.assertThrowsCause(
         "Conflicting deleted data files should throw exception",
         ValidationException.class,
         "Found conflicting deleted files that can contain records matching ref(name=\"id\") == 1:",
@@ -184,7 +184,7 @@ public class TestConflictValidation extends SparkExtensionsTestBase {
 
     // Validating from no snapshot id defaults to beginning snapshot id and finds conflicts
     Dataset<Row> conflictingDf = spark.createDataFrame(records, SimpleRecord.class);
-    AssertHelpers.assertThrows(
+    AssertHelpers.assertThrowsCause(
         "Conflicting new data files should throw exception",
         ValidationException.class,
         "Found conflicting files that can contain records matching ref(name=\"id\") == 1:",
@@ -225,7 +225,7 @@ public class TestConflictValidation extends SparkExtensionsTestBase {
     // Validating from previous snapshot finds conflicts
     List<SimpleRecord> conflictingRecords = Lists.newArrayList(new SimpleRecord(1, "a"));
     Dataset<Row> conflictingDf = spark.createDataFrame(conflictingRecords, SimpleRecord.class);
-    AssertHelpers.assertThrows(
+    AssertHelpers.assertThrowsCause(
         "Conflicting new delete files should throw exception",
         ValidationException.class,
         "Found new conflicting delete files that can apply to records matching ref(name=\"id\") == 1:",
@@ -278,7 +278,7 @@ public class TestConflictValidation extends SparkExtensionsTestBase {
 
     // Validating from previous snapshot finds conflicts
     Dataset<Row> conflictingDf = spark.createDataFrame(records, SimpleRecord.class);
-    AssertHelpers.assertThrows(
+    AssertHelpers.assertThrowsCause(
         "Conflicting deleted data files should throw exception",
         ValidationException.class,
         "Found conflicting files that can contain records matching partitions [id=1]",
@@ -318,7 +318,7 @@ public class TestConflictValidation extends SparkExtensionsTestBase {
 
     // Validating from previous snapshot finds conflicts
     Dataset<Row> conflictingDf = spark.createDataFrame(records, SimpleRecord.class);
-    AssertHelpers.assertThrows(
+    AssertHelpers.assertThrowsCause(
         "Conflicting deleted data files should throw exception",
         ValidationException.class,
         "Found new conflicting delete files that can apply to records matching [id=1]",
@@ -357,7 +357,7 @@ public class TestConflictValidation extends SparkExtensionsTestBase {
     spark.createDataFrame(records, SimpleRecord.class).coalesce(1).writeTo(tableName).append();
     Dataset<Row> conflictingDf = spark.createDataFrame(records, SimpleRecord.class);
 
-    AssertHelpers.assertThrows(
+    AssertHelpers.assertThrowsCause(
         "Conflicting deleted data files should throw exception",
         ValidationException.class,
         "Found conflicting deleted files that can apply to records matching [id=1]",
@@ -409,7 +409,7 @@ public class TestConflictValidation extends SparkExtensionsTestBase {
 
     // Validating from null snapshot is equivalent to validating from beginning
     Dataset<Row> conflictingDf = spark.createDataFrame(records, SimpleRecord.class);
-    AssertHelpers.assertThrows(
+    AssertHelpers.assertThrowsCause(
         "Conflicting deleted data files should throw exception",
         ValidationException.class,
         "Found conflicting files that can contain records matching partitions [id=1]",
