@@ -25,8 +25,10 @@ import java.io.IOException;
 import java.util.Map;
 import java.util.concurrent.atomic.AtomicInteger;
 import org.apache.iceberg.Schema;
+import org.apache.iceberg.relocated.com.google.common.collect.ImmutableMap;
 import org.apache.iceberg.relocated.com.google.common.collect.Lists;
 import org.apache.iceberg.relocated.com.google.common.collect.Maps;
+import org.apache.iceberg.spark.SparkSQLProperties;
 import org.apache.iceberg.types.TypeUtil;
 import org.apache.iceberg.types.Types;
 import org.apache.iceberg.types.Types.ListType;
@@ -229,13 +231,17 @@ public abstract class AvroDataTest {
 
   @Test
   public void testTimestampWithoutZone() throws IOException {
-    Schema schema =
-        TypeUtil.assignIncreasingFreshIds(
-            new Schema(
-                required(0, "id", LongType.get()),
-                optional(1, "ts_without_zone", Types.TimestampType.withoutZone())));
+    withSQLConf(
+        ImmutableMap.of(SparkSQLProperties.HANDLE_TIMESTAMP_WITHOUT_TIMEZONE, "true"),
+        () -> {
+          Schema schema =
+              TypeUtil.assignIncreasingFreshIds(
+                  new Schema(
+                      required(0, "id", LongType.get()),
+                      optional(1, "ts_without_zone", Types.TimestampType.withoutZone())));
 
-    writeAndValidate(schema);
+          writeAndValidate(schema);
+        });
   }
 
   protected void withSQLConf(Map<String, String> conf, Action action) throws IOException {
