@@ -18,8 +18,6 @@
  */
 package org.apache.iceberg.spark.extensions;
 
-import static org.apache.iceberg.types.Types.NestedField.optional;
-
 import java.io.File;
 import java.io.IOException;
 import java.util.List;
@@ -34,20 +32,10 @@ import org.apache.avro.generic.GenericData;
 import org.apache.avro.generic.GenericDatumWriter;
 import org.apache.avro.generic.GenericRecord;
 import org.apache.avro.io.DatumWriter;
-import org.apache.iceberg.AssertHelpers;
 import org.apache.iceberg.DataFile;
-import org.apache.iceberg.Files;
-import org.apache.iceberg.MetadataTableType;
-import org.apache.iceberg.MetricsConfig;
 import org.apache.iceberg.TableProperties;
-import org.apache.iceberg.data.Record;
-import org.apache.iceberg.data.orc.GenericOrcWriter;
-import org.apache.iceberg.io.FileAppender;
-import org.apache.iceberg.io.OutputFile;
 import org.apache.iceberg.relocated.com.google.common.collect.ImmutableList;
-import org.apache.iceberg.relocated.com.google.common.collect.ImmutableMap;
 import org.apache.iceberg.relocated.com.google.common.collect.Lists;
-import org.apache.iceberg.types.Types;
 import org.apache.spark.sql.Dataset;
 import org.apache.spark.sql.Row;
 import org.apache.spark.sql.RowFactory;
@@ -55,6 +43,8 @@ import org.apache.spark.sql.types.DataTypes;
 import org.apache.spark.sql.types.Metadata;
 import org.apache.spark.sql.types.StructField;
 import org.apache.spark.sql.types.StructType;
+import org.assertj.core.api.Assertions;
+import org.joda.time.DateTime;
 import org.junit.After;
 import org.junit.Assert;
 import org.junit.Assume;
@@ -100,12 +90,41 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
 
     sql(createIceberg, tableName);
 
-    Object result =
-        scalarSql(
+    List<Object[]> result =
+        sql(
             "CALL %s.system.add_files('%s', '`parquet`.`%s`')",
             catalogName, tableName, fileTableDir.getAbsolutePath());
 
-    Assert.assertEquals(2L, result);
+    assertEquals("Procedure output must match", ImmutableList.of(row(2L, 1L)), result);
+
+    assertEquals(
+        "Iceberg table contains correct data",
+        sql("SELECT * FROM %s ORDER BY id", sourceTableName),
+        sql("SELECT * FROM %s ORDER BY id", tableName));
+  }
+
+  @Test
+  public void deleteAndAddBackUnpartitioned() {
+    createUnpartitionedFileTable("parquet");
+
+    String createIceberg =
+        "CREATE TABLE %s (id Integer, name String, dept String, subdept String) USING iceberg";
+
+    sql(createIceberg, tableName);
+
+    sql(
+        "CALL %s.system.add_files('%s', '`parquet`.`%s`')",
+        catalogName, tableName, fileTableDir.getAbsolutePath());
+
+    String deleteData = "DELETE FROM %s";
+    sql(deleteData, tableName);
+
+    List<Object[]> result =
+        sql(
+            "CALL %s.system.add_files('%s', '`parquet`.`%s`')",
+            catalogName, tableName, fileTableDir.getAbsolutePath());
+
+    assertEquals("Procedure output must match", ImmutableList.of(row(2L, 1L)), result);
 
     assertEquals(
         "Iceberg table contains correct data",
@@ -167,11 +186,11 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
     String createIceberg = "CREATE TABLE %s (id Long, data String) USING iceberg";
     sql(createIceberg, tableName);
 
-    Object result =
-        scalarSql(
+    List<Object[]> result =
+        sql(
             "CALL %s.system.add_files('%s', '`avro`.`%s`')",
             catalogName, tableName, outputFile.getPath());
-    Assert.assertEquals(1L, result);
+    assertEquals("Procedure output must match", ImmutableList.of(row(1L, 1L)), result);
 
     List<Object[]> expected = Lists.newArrayList(new Object[] {1L, "a"}, new Object[] {2L, "b"});
 
@@ -222,10 +241,10 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
 
     sql(createIceberg, tableName);
 
-    Object result =
-        scalarSql("CALL %s.system.add_files('%s', '%s')", catalogName, tableName, sourceTableName);
+    List<Object[]> result =
+        sql("CALL %s.system.add_files('%s', '%s')", catalogName, tableName, sourceTableName);
 
-    Assert.assertEquals(2L, result);
+    assertEquals("Procedure output must match", ImmutableList.of(row(2L, 1L)), result);
 
     assertEquals(
         "Iceberg table contains correct data",
@@ -242,12 +261,12 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
 
     sql(createIceberg, tableName);
 
-    Object result =
-        scalarSql(
+    List<Object[]> result =
+        sql(
             "CALL %s.system.add_files('%s', '`parquet`.`%s`')",
             catalogName, tableName, fileTableDir.getAbsolutePath());
 
-    Assert.assertEquals(2L, result);
+    assertEquals("Procedure output must match", ImmutableList.of(row(2L, 1L)), result);
 
     assertEquals(
         "Iceberg table contains correct data",
@@ -263,12 +282,12 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
 
     sql(createIceberg, tableName);
 
-    Object result =
-        scalarSql(
+    List<Object[]> result =
+        sql(
             "CALL %s.system.add_files('%s', '`parquet`.`%s`')",
             catalogName, tableName, fileTableDir.getAbsolutePath());
 
-    Assert.assertEquals(2L, result);
+    assertEquals("Procedure output must match", ImmutableList.of(row(2L, 1L)), result);
 
     assertEquals(
         "Iceberg table contains correct data",
@@ -285,12 +304,12 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
 
     sql(createIceberg, tableName);
 
-    Object result =
-        scalarSql(
+    List<Object[]> result =
+        sql(
             "CALL %s.system.add_files('%s', '`parquet`.`%s`')",
             catalogName, tableName, fileTableDir.getAbsolutePath());
 
-    Assert.assertEquals(8L, result);
+    assertEquals("Procedure output must match", ImmutableList.of(row(8L, 4L)), result);
 
     assertEquals(
         "Iceberg table contains correct data",
@@ -307,12 +326,12 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
 
     sql(createIceberg, tableName);
 
-    Object result =
-        scalarSql(
+    List<Object[]> result =
+        sql(
             "CALL %s.system.add_files('%s', '`parquet`.`%s`')",
             catalogName, tableName, fileTableDir.getAbsolutePath());
 
-    Assert.assertEquals(8L, result);
+    assertEquals("Procedure output must match", ImmutableList.of(row(8L, 4L)), result);
 
     assertEquals(
         "Iceberg table contains correct data",
@@ -374,10 +393,10 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
 
     sql(createIceberg, tableName);
 
-    Object result =
-        scalarSql("CALL %s.system.add_files('%s', '%s')", catalogName, tableName, sourceTableName);
+    List<Object[]> result =
+        sql("CALL %s.system.add_files('%s', '%s')", catalogName, tableName, sourceTableName);
 
-    Assert.assertEquals(8L, result);
+    assertEquals("Procedure output must match", ImmutableList.of(row(8L, 4L)), result);
 
     assertEquals(
         "Iceberg table contains correct data",
@@ -394,12 +413,12 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
 
     sql(createIceberg, tableName);
 
-    Object result =
-        scalarSql(
+    List<Object[]> result =
+        sql(
             "CALL %s.system.add_files('%s', '`parquet`.`%s`', map('id', 1))",
             catalogName, tableName, fileTableDir.getAbsolutePath());
 
-    Assert.assertEquals(2L, result);
+    assertEquals("Procedure output must match", ImmutableList.of(row(2L, 1L)), result);
 
     assertEquals(
         "Iceberg table contains correct data",
@@ -407,6 +426,35 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
         sql("SELECT id, name, dept, subdept FROM %s ORDER BY id", tableName));
   }
 
+  @Test
+  public void deleteAndAddBackPartitioned() {
+    createPartitionedFileTable("parquet");
+
+    String createIceberg =
+        "CREATE TABLE %s (id Integer, name String, dept String, subdept String) USING iceberg PARTITIONED BY (id)";
+
+    sql(createIceberg, tableName);
+
+    sql(
+        "CALL %s.system.add_files('%s', '`parquet`.`%s`', map('id', 1))",
+        catalogName, tableName, fileTableDir.getAbsolutePath());
+
+    String deleteData = "DELETE FROM %s where id = 1";
+    sql(deleteData, tableName);
+
+    List<Object[]> result =
+        sql(
+            "CALL %s.system.add_files('%s', '`parquet`.`%s`', map('id', 1))",
+            catalogName, tableName, fileTableDir.getAbsolutePath());
+
+    assertEquals("Procedure output must match", ImmutableList.of(row(2L, 1L)), result);
+
+    assertEquals(
+        "Iceberg table contains correct data",
+        sql("SELECT id, name, dept, subdept FROM %s WHERE id = 1 ORDER BY id", sourceTableName),
+        sql("SELECT * FROM %s ORDER BY id", tableName));
+  }
+
   @Test
   public void addPartitionToPartitionedSnapshotIdInheritanceEnabledInTwoRuns() {
     createPartitionedFileTable("parquet");
@@ -431,20 +479,57 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
         sql("SELECT id, name, dept, subdept FROM %s ORDER BY id", tableName));
 
     // verify manifest file name has uuid pattern
-    List<Row> rows =
-        spark
-            .read()
-            .format("iceberg")
-            .load(String.format("%s.%s", tableName, MetadataTableType.MANIFESTS.name()))
-            .select("path")
-            .collectAsList();
+    String manifestPath = (String) sql("select path from %s.manifests", tableName).get(0)[0];
 
     Pattern uuidPattern = Pattern.compile("[a-f0-9]{8}(?:-[a-f0-9]{4}){4}[a-f0-9]{8}");
 
-    Matcher matcher = uuidPattern.matcher((String) rows.get(0).get(0));
+    Matcher matcher = uuidPattern.matcher(manifestPath);
     Assert.assertTrue("verify manifest path has uuid", matcher.find());
   }
 
+  @Test
+  public void addDataPartitionedByDateToPartitioned() {
+    createDatePartitionedFileTable("parquet");
+
+    String createIceberg =
+        "CREATE TABLE %s (id Integer, name String, date Date) USING iceberg PARTITIONED BY (date)";
+
+    sql(createIceberg, tableName);
+
+    List<Object[]> result =
+        sql(
+            "CALL %s.system.add_files('%s', '`parquet`.`%s`', map('date', '2021-01-01'))",
+            catalogName, tableName, fileTableDir.getAbsolutePath());
+
+    assertEquals("Procedure output must match", ImmutableList.of(row(2L, 1L)), result);
+
+    assertEquals(
+        "Iceberg table contains correct data",
+        sql("SELECT id, name, date FROM %s WHERE date = '2021-01-01' ORDER BY id", sourceTableName),
+        sql("SELECT id, name, date FROM %s ORDER BY id", tableName));
+  }
+
+  @Test
+  public void addDataPartitionedVerifyPartitionTypeInferredCorrectly() {
+    createTableWithTwoPartitions("parquet");
+
+    String createIceberg =
+        "CREATE TABLE %s (id Integer, name String, date Date, dept String) USING iceberg PARTITIONED BY (date, dept)";
+
+    sql(createIceberg, tableName);
+
+    sql(
+        "CALL %s.system.add_files('%s', '`parquet`.`%s`', map('date', '2021-01-01'))",
+        catalogName, tableName, fileTableDir.getAbsolutePath());
+
+    String sqlFormat =
+        "SELECT id, name, dept, date FROM %s WHERE date = '2021-01-01' and dept= '01' ORDER BY id";
+    assertEquals(
+        "Iceberg table contains correct data",
+        sql(sqlFormat, sourceTableName),
+        sql(sqlFormat, tableName));
+  }
+
   @Test
   public void addFilteredPartitionsToPartitioned() {
     createCompositePartitionedTable("parquet");
@@ -455,12 +540,12 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
 
     sql(createIceberg, tableName);
 
-    Object result =
-        scalarSql(
+    List<Object[]> result =
+        sql(
             "CALL %s.system.add_files('%s', '`parquet`.`%s`', map('id', 1))",
             catalogName, tableName, fileTableDir.getAbsolutePath());
 
-    Assert.assertEquals(2L, result);
+    assertEquals("Procedure output must match", ImmutableList.of(row(2L, 1L)), result);
 
     assertEquals(
         "Iceberg table contains correct data",
@@ -478,12 +563,12 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
 
     sql(createIceberg, tableName);
 
-    Object result =
-        scalarSql(
+    List<Object[]> result =
+        sql(
             "CALL %s.system.add_files('%s', '`parquet`.`%s`', map('dept', 'hr'))",
             catalogName, tableName, fileTableDir.getAbsolutePath());
 
-    Assert.assertEquals(6L, result);
+    assertEquals("Procedure output must match", ImmutableList.of(row(6L, 3L)), result);
 
     assertEquals(
         "Iceberg table contains correct data",
@@ -503,12 +588,12 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
 
     sql(createIceberg, tableName);
 
-    Object result =
-        scalarSql(
+    List<Object[]> result =
+        sql(
             "CALL %s.system.add_files('%s', '`parquet`.`%s`', map('id', 1))",
             catalogName, tableName, fileTableDir.getAbsolutePath());
 
-    Assert.assertEquals(2L, result);
+    assertEquals("Procedure output must match", ImmutableList.of(row(2L, 1L)), result);
 
     assertEquals(
         "Iceberg table contains correct data",
@@ -526,12 +611,12 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
 
     sql(createIceberg, tableName);
 
-    Object result =
-        scalarSql(
+    List<Object[]> result =
+        sql(
             "CALL %s.system.add_files('%s', '`parquet`.`%s`', map('dept', 'hr'))",
             catalogName, tableName, fileTableDir.getAbsolutePath());
 
-    Assert.assertEquals(6L, result);
+    assertEquals("Procedure output must match", ImmutableList.of(row(6L, 3L)), result);
 
     assertEquals(
         "Iceberg table contains correct data",
@@ -551,12 +636,12 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
 
     sql(createIceberg, tableName);
 
-    Object result =
-        scalarSql(
+    List<Object[]> result =
+        sql(
             "CALL %s.system.add_files('%s', '%s', map('naMe', 'John Doe'))",
             catalogName, tableName, sourceTableName);
 
-    Assert.assertEquals(2L, result);
+    assertEquals("Procedure output must match", ImmutableList.of(row(2L, 1L)), result);
 
     /*
     While we would like to use
@@ -601,12 +686,12 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
 
     sql(createIceberg, tableName);
 
-    Object result =
-        scalarSql(
+    List<Object[]> result =
+        sql(
             "CALL %s.system.add_files('%s', '%s', map('id', 1))",
             catalogName, tableName, sourceTableName);
 
-    Assert.assertEquals(2L, result);
+    assertEquals("Procedure output must match", ImmutableList.of(row(2L, 1L)), result);
 
     assertEquals(
         "Iceberg table contains correct data",
@@ -623,23 +708,21 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
 
     sql(createIceberg, tableName);
 
-    AssertHelpers.assertThrows(
-        "Should forbid adding of partitioned data to unpartitioned table",
-        IllegalArgumentException.class,
-        "Cannot use partition filter with an unpartitioned table",
-        () ->
-            scalarSql(
-                "CALL %s.system.add_files('%s', '`parquet`.`%s`', map('id', 1))",
-                catalogName, tableName, fileTableDir.getAbsolutePath()));
+    Assertions.assertThatThrownBy(
+            () ->
+                scalarSql(
+                    "CALL %s.system.add_files('%s', '`parquet`.`%s`', map('id', 1))",
+                    catalogName, tableName, fileTableDir.getAbsolutePath()))
+        .isInstanceOf(IllegalArgumentException.class)
+        .hasMessageStartingWith("Cannot use partition filter with an unpartitioned table");
 
-    AssertHelpers.assertThrows(
-        "Should forbid adding of partitioned data to unpartitioned table",
-        IllegalArgumentException.class,
-        "Cannot add partitioned files to an unpartitioned table",
-        () ->
-            scalarSql(
-                "CALL %s.system.add_files('%s', '`parquet`.`%s`')",
-                catalogName, tableName, fileTableDir.getAbsolutePath()));
+    Assertions.assertThatThrownBy(
+            () ->
+                scalarSql(
+                    "CALL %s.system.add_files('%s', '`parquet`.`%s`')",
+                    catalogName, tableName, fileTableDir.getAbsolutePath()))
+        .isInstanceOf(IllegalArgumentException.class)
+        .hasMessageStartingWith("Cannot add partitioned files to an unpartitioned table");
   }
 
   @Test
@@ -651,23 +734,24 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
 
     sql(createIceberg, tableName);
 
-    AssertHelpers.assertThrows(
-        "Should forbid adding with a mismatching partition spec",
-        IllegalArgumentException.class,
-        "is greater than the number of partitioned columns",
-        () ->
-            scalarSql(
-                "CALL %s.system.add_files('%s', '`parquet`.`%s`', map('x', '1', 'y', '2'))",
-                catalogName, tableName, fileTableDir.getAbsolutePath()));
+    Assertions.assertThatThrownBy(
+            () ->
+                scalarSql(
+                    "CALL %s.system.add_files('%s', '`parquet`.`%s`', map('x', '1', 'y', '2'))",
+                    catalogName, tableName, fileTableDir.getAbsolutePath()))
+        .isInstanceOf(IllegalArgumentException.class)
+        .hasMessageStartingWith("Cannot add data files to target table")
+        .hasMessageContaining("is greater than the number of partitioned columns");
 
-    AssertHelpers.assertThrows(
-        "Should forbid adding with partition spec with incorrect columns",
-        IllegalArgumentException.class,
-        "specified partition filter refers to columns that are not partitioned",
-        () ->
-            scalarSql(
-                "CALL %s.system.add_files('%s', '`parquet`.`%s`', map('dept', '2'))",
-                catalogName, tableName, fileTableDir.getAbsolutePath()));
+    Assertions.assertThatThrownBy(
+            () ->
+                scalarSql(
+                    "CALL %s.system.add_files('%s', '`parquet`.`%s`', map('dept', '2'))",
+                    catalogName, tableName, fileTableDir.getAbsolutePath()))
+        .isInstanceOf(IllegalArgumentException.class)
+        .hasMessageStartingWith("Cannot add files to target table")
+        .hasMessageContaining(
+            "specified partition filter refers to columns that are not partitioned");
   }
 
   @Test
@@ -679,23 +763,23 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
 
     sql(createIceberg, tableName);
 
-    Object result1 =
-        scalarSql(
+    List<Object[]> result1 =
+        sql(
             "CALL %s.system.add_files("
                 + "table => '%s', "
                 + "source_table => '%s', "
                 + "partition_filter => map('id', 1))",
             catalogName, tableName, sourceTableName);
-    Assert.assertEquals(2L, result1);
+    assertEquals("Procedure output must match", ImmutableList.of(row(2L, 1L)), result1);
 
-    Object result2 =
-        scalarSql(
+    List<Object[]> result2 =
+        sql(
             "CALL %s.system.add_files("
                 + "table => '%s', "
                 + "source_table => '%s', "
                 + "partition_filter => map('id', 2))",
             catalogName, tableName, sourceTableName);
-    Assert.assertEquals(2L, result2);
+    assertEquals("Procedure output must match", ImmutableList.of(row(2L, 1L)), result2);
 
     assertEquals(
         "Iceberg table contains correct data",
@@ -716,25 +800,25 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
 
     sql(createIceberg, tableName);
 
-    scalarSql(
+    sql(
         "CALL %s.system.add_files("
             + "table => '%s', "
             + "source_table => '%s', "
             + "partition_filter => map('id', 1))",
         catalogName, tableName, sourceTableName);
 
-    AssertHelpers.assertThrows(
-        "Should not allow adding duplicate files",
-        IllegalStateException.class,
-        "Cannot complete import because data files to be imported already"
-            + " exist within the target table",
-        () ->
-            scalarSql(
-                "CALL %s.system.add_files("
-                    + "table => '%s', "
-                    + "source_table => '%s', "
-                    + "partition_filter => map('id', 1))",
-                catalogName, tableName, sourceTableName));
+    Assertions.assertThatThrownBy(
+            () ->
+                scalarSql(
+                    "CALL %s.system.add_files("
+                        + "table => '%s', "
+                        + "source_table => '%s', "
+                        + "partition_filter => map('id', 1))",
+                    catalogName, tableName, sourceTableName))
+        .isInstanceOf(IllegalStateException.class)
+        .hasMessageStartingWith(
+            "Cannot complete import because data files to be imported already"
+                + " exist within the target table");
   }
 
   @Test
@@ -746,18 +830,18 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
 
     sql(createIceberg, tableName);
 
-    Object result1 =
-        scalarSql(
+    List<Object[]> result1 =
+        sql(
             "CALL %s.system.add_files("
                 + "table => '%s', "
                 + "source_table => '%s', "
                 + "partition_filter => map('id', 1))",
             catalogName, tableName, sourceTableName);
 
-    Assert.assertEquals(2L, result1);
+    assertEquals("Procedure output must match", ImmutableList.of(row(2L, 1L)), result1);
 
-    Object result2 =
-        scalarSql(
+    List<Object[]> result2 =
+        sql(
             "CALL %s.system.add_files("
                 + "table => '%s', "
                 + "source_table => '%s', "
@@ -765,7 +849,7 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
                 + "check_duplicate_files => false)",
             catalogName, tableName, sourceTableName);
 
-    Assert.assertEquals(2L, result2);
+    assertEquals("Procedure output must match", ImmutableList.of(row(2L, 1L)), result2);
 
     assertEquals(
         "Iceberg table contains correct data",
@@ -785,16 +869,17 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
 
     sql(createIceberg, tableName);
 
-    scalarSql("CALL %s.system.add_files('%s', '%s')", catalogName, tableName, sourceTableName);
+    sql("CALL %s.system.add_files('%s', '%s')", catalogName, tableName, sourceTableName);
 
-    AssertHelpers.assertThrows(
-        "Should not allow adding duplicate files",
-        IllegalStateException.class,
-        "Cannot complete import because data files to be imported already"
-            + " exist within the target table",
-        () ->
-            scalarSql(
-                "CALL %s.system.add_files('%s', '%s')", catalogName, tableName, sourceTableName));
+    Assertions.assertThatThrownBy(
+            () ->
+                scalarSql(
+                    "CALL %s.system.add_files('%s', '%s')",
+                    catalogName, tableName, sourceTableName))
+        .isInstanceOf(IllegalStateException.class)
+        .hasMessageStartingWith(
+            "Cannot complete import because data files to be imported already"
+                + " exist within the target table");
   }
 
   @Test
@@ -806,18 +891,18 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
 
     sql(createIceberg, tableName);
 
-    Object result1 =
-        scalarSql("CALL %s.system.add_files('%s', '%s')", catalogName, tableName, sourceTableName);
-    Assert.assertEquals(2L, result1);
+    List<Object[]> result1 =
+        sql("CALL %s.system.add_files('%s', '%s')", catalogName, tableName, sourceTableName);
+    assertEquals("Procedure output must match", ImmutableList.of(row(2L, 1L)), result1);
 
-    Object result2 =
-        scalarSql(
+    List<Object[]> result2 =
+        sql(
             "CALL %s.system.add_files("
                 + "table => '%s', "
                 + "source_table => '%s',"
                 + "check_duplicate_files => false)",
             catalogName, tableName, sourceTableName);
-    Assert.assertEquals(2L, result2);
+    assertEquals("Procedure output must match", ImmutableList.of(row(2L, 1L)), result2);
 
     assertEquals(
         "Iceberg table contains correct data",
@@ -827,46 +912,6 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
         sql("SELECT * FROM %s ORDER BY id", tableName));
   }
 
-  @Test
-  public void addOrcFileWithDoubleAndFloatColumns() throws Exception {
-    // Spark Session Catalog cannot load metadata tables
-    // with "The namespace in session catalog must have exactly one name part"
-    Assume.assumeFalse(catalogName.equals("spark_catalog"));
-
-    // Create an ORC file
-    File outputFile = temp.newFile("test.orc");
-    final int numRows = 5;
-    List<Record> expectedRecords = createOrcFile(outputFile, numRows);
-    String createIceberg = "CREATE TABLE %s (x float, y double, z long) USING iceberg";
-    sql(createIceberg, tableName);
-
-    Object result =
-        scalarSql(
-            "CALL %s.system.add_files('%s', '`orc`.`%s`')",
-            catalogName, tableName, outputFile.getPath());
-    Assert.assertEquals(1L, result);
-
-    List<Object[]> expected =
-        expectedRecords.stream()
-            .map(record -> new Object[] {record.get(0), record.get(1), record.get(2)})
-            .collect(Collectors.toList());
-
-    // x goes 2.00, 1.99, 1.98, ...
-    assertEquals(
-        "Iceberg table contains correct data",
-        expected,
-        sql("SELECT * FROM %s ORDER BY x DESC", tableName));
-
-    List<Object[]> actualRecordCount =
-        sql("select %s from %s.files", DataFile.RECORD_COUNT.name(), tableName);
-    List<Object[]> expectedRecordCount = Lists.newArrayList();
-    expectedRecordCount.add(new Object[] {(long) numRows});
-    assertEquals(
-        "Iceberg file metadata should have correct metadata count",
-        expectedRecordCount,
-        actualRecordCount);
-  }
-
   @Test
   public void testEmptyImportDoesNotThrow() {
 
@@ -875,11 +920,11 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
     sql(createIceberg, tableName);
 
     // Empty path based import
-    Object pathResult =
-        scalarSql(
+    List<Object[]> pathResult =
+        sql(
             "CALL %s.system.add_files('%s', '`parquet`.`%s`')",
             catalogName, tableName, fileTableDir.getAbsolutePath());
-    Assert.assertEquals(0L, pathResult);
+    assertEquals("Procedure output must match", ImmutableList.of(row(0L, 0L)), pathResult);
     assertEquals(
         "Iceberg table contains no added data when importing from an empty path",
         emptyQueryResult,
@@ -890,9 +935,9 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
         "CREATE TABLE %s (id Integer, name String, dept String, subdept String) STORED AS parquet";
     sql(createHive, sourceTableName);
 
-    Object tableResult =
-        scalarSql("CALL %s.system.add_files('%s', '%s')", catalogName, tableName, sourceTableName);
-    Assert.assertEquals(0L, tableResult);
+    List<Object[]> tableResult =
+        sql("CALL %s.system.add_files('%s', '%s')", catalogName, tableName, sourceTableName);
+    assertEquals("Procedure output must match", ImmutableList.of(row(0L, 0L)), tableResult);
     assertEquals(
         "Iceberg table contains no added data when importing from an empty table",
         emptyQueryResult,
@@ -914,15 +959,15 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
 
     sql(createIceberg, tableName);
 
-    Object tableResult =
-        scalarSql(
+    List<Object[]> tableResult =
+        sql(
             "CALL %s.system.add_files("
                 + "table => '%s', "
                 + "source_table => '%s', "
                 + "partition_filter => map('id', %d))",
             catalogName, tableName, sourceTableName, emptyPartitionId);
 
-    Assert.assertEquals(0L, tableResult);
+    assertEquals("Procedure output must match", ImmutableList.of(row(0L, 0L)), tableResult);
     assertEquals(
         "Iceberg table contains no added data when importing from an empty table",
         emptyQueryResult,
@@ -971,6 +1016,28 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
           unpartitionedDF.col("dept"),
           unpartitionedDF.col("name").as("naMe"));
 
+  private static final StructField[] dateStruct = {
+    new StructField("id", DataTypes.IntegerType, true, Metadata.empty()),
+    new StructField("name", DataTypes.StringType, true, Metadata.empty()),
+    new StructField("ts", DataTypes.DateType, true, Metadata.empty()),
+    new StructField("dept", DataTypes.StringType, true, Metadata.empty()),
+  };
+
+  private static java.sql.Date toDate(String value) {
+    return new java.sql.Date(DateTime.parse(value).getMillis());
+  }
+
+  private static final Dataset<Row> dateDF =
+      spark
+          .createDataFrame(
+              ImmutableList.of(
+                  RowFactory.create(1, "John Doe", toDate("2021-01-01"), "01"),
+                  RowFactory.create(2, "Jane Doe", toDate("2021-01-01"), "01"),
+                  RowFactory.create(3, "Matt Doe", toDate("2021-01-02"), "02"),
+                  RowFactory.create(4, "Will Doe", toDate("2021-01-02"), "02")),
+              new StructType(dateStruct))
+          .repartition(2);
+
   private void createUnpartitionedFileTable(String format) {
     String createParquet =
         "CREATE TABLE %s (id Integer, name String, dept String, subdept String) USING %s LOCATION '%s'";
@@ -1049,42 +1116,23 @@ public class TestAddFilesProcedure extends SparkExtensionsTestBase {
     partitionedDF.write().insertInto(sourceTableName);
   }
 
-  // Update this to not write a file for import using Iceberg's ID numbers
-  public List<Record> createOrcFile(File orcFile, int numRows) throws IOException {
-    // Needs to be deleted but depend on the rule to delete the file for us again at the end.
-    if (orcFile.exists()) {
-      orcFile.delete();
-    }
-    final org.apache.iceberg.Schema icebergSchema =
-        new org.apache.iceberg.Schema(
-            optional(1, "x", Types.FloatType.get()),
-            optional(2, "y", Types.DoubleType.get()),
-            optional(3, "z", Types.LongType.get()));
+  private void createDatePartitionedFileTable(String format) {
+    String createParquet =
+        "CREATE TABLE %s (id Integer, name String, date Date) USING %s "
+            + "PARTITIONED BY (date) LOCATION '%s'";
 
-    List<Record> records = Lists.newArrayListWithExpectedSize(numRows);
-    for (int i = 0; i < numRows; i += 1) {
-      Record record = org.apache.iceberg.data.GenericRecord.create(icebergSchema);
-      record.setField("x", ((float) (100 - i)) / 100F + 1.0F); // 2.0f, 1.99f, 1.98f, ...
-      record.setField("y", ((double) i) / 100.0D + 2.0D); // 2.0d, 2.01d, 2.02d, ...
-      record.setField("z", 5_000_000_000L + i);
-      records.add(record);
-    }
+    sql(createParquet, sourceTableName, format, fileTableDir.getAbsolutePath());
 
-    OutputFile outFile = Files.localOutput(orcFile);
-    try (FileAppender<Record> appender =
-        org.apache
-            .iceberg
-            .orc
-            .ORC
-            .write(outFile)
-            .schema(icebergSchema)
-            .metricsConfig(
-                MetricsConfig.fromProperties(
-                    ImmutableMap.of("write.metadata.metrics.default", "none")))
-            .createWriterFunc(GenericOrcWriter::buildWriter)
-            .build()) {
-      appender.addAll(records);
-    }
-    return records;
+    dateDF.select("id", "name", "ts").write().insertInto(sourceTableName);
+  }
+
+  private void createTableWithTwoPartitions(String format) {
+    String createParquet =
+        "CREATE TABLE %s (id Integer, name String, date Date, dept String) USING %s "
+            + "PARTITIONED BY (date, dept) LOCATION '%s'";
+
+    sql(createParquet, sourceTableName, format, fileTableDir.getAbsolutePath());
+
+    dateDF.write().insertInto(sourceTableName);
   }
 }
