@@ -20,16 +20,15 @@ package org.apache.iceberg.spark.source;
 
 import java.util.List;
 import java.util.Map;
-import org.apache.iceberg.AssertHelpers;
 import org.apache.iceberg.Table;
 import org.apache.iceberg.TableProperties;
 import org.apache.iceberg.relocated.com.google.common.collect.ImmutableList;
 import org.apache.iceberg.spark.SparkCatalogTestBase;
 import org.apache.iceberg.spark.SparkWriteOptions;
-import org.apache.spark.SparkException;
 import org.apache.spark.sql.Dataset;
 import org.apache.spark.sql.Row;
 import org.apache.spark.sql.catalyst.analysis.NoSuchTableException;
+import org.assertj.core.api.Assertions;
 import org.junit.After;
 import org.junit.Test;
 
@@ -163,20 +162,17 @@ public class TestRequiredDistributionAndOrdering extends SparkCatalogTestBase {
     Dataset<Row> inputDF = ds.coalesce(1).sortWithinPartitions("c1");
 
     // should fail if ordering is disabled
-    AssertHelpers.assertThrows(
-        "Should reject writes without ordering",
-        SparkException.class,
-        "Writing job aborted",
-        () -> {
-          try {
-            inputDF
-                .writeTo(tableName)
-                .option(SparkWriteOptions.USE_TABLE_DISTRIBUTION_AND_ORDERING, "false")
-                .append();
-          } catch (NoSuchTableException e) {
-            throw new RuntimeException(e);
-          }
-        });
+    Assertions.assertThatThrownBy(
+            () ->
+                inputDF
+                    .writeTo(tableName)
+                    .option(SparkWriteOptions.USE_TABLE_DISTRIBUTION_AND_ORDERING, "false")
+                    .append())
+        .cause()
+        .isInstanceOf(IllegalStateException.class)
+        .hasMessageStartingWith(
+            "Incoming records violate the writer assumption that records are clustered by spec "
+                + "and by partition within each spec. Either cluster the incoming records or switch to fanout writers.");
   }
 
   @Test
@@ -216,7 +212,7 @@ public class TestRequiredDistributionAndOrdering extends SparkCatalogTestBase {
   }
 
   @Test
-  public void testNoSortBucketTransformsWithoutExtensions() throws NoSuchTableException {
+  public void testSortBucketTransformsWithoutExtensions() throws NoSuchTableException {
     sql(
         "CREATE TABLE %s (c1 INT, c2 STRING, c3 STRING) "
             + "USING iceberg "
@@ -232,20 +228,7 @@ public class TestRequiredDistributionAndOrdering extends SparkCatalogTestBase {
     Dataset<Row> ds = spark.createDataFrame(data, ThreeColumnRecord.class);
     Dataset<Row> inputDF = ds.coalesce(1).sortWithinPartitions("c1");
 
-    // should fail by default as extensions are disabled
-    AssertHelpers.assertThrows(
-        "Should reject writes without ordering",
-        SparkException.class,
-        "Writing job aborted",
-        () -> {
-          try {
-            inputDF.writeTo(tableName).append();
-          } catch (NoSuchTableException e) {
-            throw new RuntimeException(e);
-          }
-        });
-
-    inputDF.writeTo(tableName).option(SparkWriteOptions.FANOUT_ENABLED, "true").append();
+    inputDF.writeTo(tableName).append();
 
     List<Object[]> expected =
         ImmutableList.of(
