@@ -51,6 +51,8 @@ import org.apache.iceberg.io.ClusteredDataWriter;
 import org.apache.iceberg.io.ClusteredPositionDeleteWriter;
 import org.apache.iceberg.io.DataWriteResult;
 import org.apache.iceberg.io.DeleteWriteResult;
+import org.apache.iceberg.io.FanoutDataWriter;
+import org.apache.iceberg.io.FanoutPositionOnlyDeleteWriter;
 import org.apache.iceberg.io.FileIO;
 import org.apache.iceberg.io.OutputFileFactory;
 import org.apache.iceberg.io.PartitioningWriter;
@@ -93,13 +95,14 @@ class SparkPositionDeltaWrite implements DeltaWrite, RequiresDistributionAndOrde
   private final Command command;
   private final SparkBatchQueryScan scan;
   private final IsolationLevel isolationLevel;
-  private final Context context;
   private final String applicationId;
   private final boolean wapEnabled;
   private final String wapId;
   private final String branch;
   private final Map<String, String> extraSnapshotMetadata;
   private final SparkWriteRequirements writeRequirements;
+  private final Context context;
+  private final Map<String, String> writeProperties;
 
   private boolean cleanupOnAbort = true;
 
@@ -117,18 +120,21 @@ class SparkPositionDeltaWrite implements DeltaWrite, RequiresDistributionAndOrde
     this.command = command;
     this.scan = scan;
     this.isolationLevel = isolationLevel;
-    this.context = new Context(dataSchema, writeConf, info);
     this.applicationId = spark.sparkContext().applicationId();
     this.wapEnabled = writeConf.wapEnabled();
     this.wapId = writeConf.wapId();
     this.branch = writeConf.branch();
     this.extraSnapshotMetadata = writeConf.extraSnapshotMetadata();
     this.writeRequirements = writeConf.positionDeltaRequirements(command);
+    this.context = new Context(dataSchema, writeConf, info, writeRequirements);
+    this.writeProperties = writeConf.writeProperties();
   }
 
   @Override
   public Distribution requiredDistribution() {
-    return writeRequirements.distribution();
+    Distribution distribution = writeRequirements.distribution();
+    LOG.info("Requesting {} as write distribution for table {}", distribution, table.name());
+    return distribution;
   }
 
   @Override
@@ -138,7 +144,16 @@ class SparkPositionDeltaWrite implements DeltaWrite, RequiresDistributionAndOrde
 
   @Override
   public SortOrder[] requiredOrdering() {
-    return writeRequirements.ordering();
+    SortOrder[] ordering = writeRequirements.ordering();
+    LOG.info("Requesting {} as write ordering for table {}", ordering, table.name());
+    return ordering;
+  }
+
+  @Override
+  public long advisoryPartitionSizeInBytes() {
+    long size = writeRequirements.advisoryPartitionSize();
+    LOG.info("Requesting {} bytes advisory partition size for table {}", size, table.name());
+    return size;
   }
 
   @Override
@@ -153,7 +168,7 @@ class SparkPositionDeltaWrite implements DeltaWrite, RequiresDistributionAndOrde
       // broadcast the table metadata as the writer factory will be sent to executors
       Broadcast<Table> tableBroadcast =
           sparkContext.broadcast(SerializableTableWithSize.copyOf(table));
-      return new PositionDeltaWriteFactory(tableBroadcast, command, context);
+      return new PositionDeltaWriteFactory(tableBroadcast, command, context, writeProperties);
     }
 
     @Override
@@ -324,11 +339,17 @@ class SparkPositionDeltaWrite implements DeltaWrite, RequiresDistributionAndOrde
     private final Broadcast<Table> tableBroadcast;
     private final Command command;
     private final Context context;
+    private final Map<String, String> writeProperties;
 
-    PositionDeltaWriteFactory(Broadcast<Table> tableBroadcast, Command command, Context context) {
+    PositionDeltaWriteFactory(
+        Broadcast<Table> tableBroadcast,
+        Command command,
+        Context context,
+        Map<String, String> writeProperties) {
       this.tableBroadcast = tableBroadcast;
       this.command = command;
       this.context = context;
+      this.writeProperties = writeProperties;
     }
 
     @Override
@@ -354,6 +375,7 @@ class SparkPositionDeltaWrite implements DeltaWrite, RequiresDistributionAndOrde
               .dataSparkType(context.dataSparkType())
               .deleteFileFormat(context.deleteFileFormat())
               .positionDeleteSparkType(context.deleteSparkType())
+              .writeProperties(writeProperties)
               .build();
 
       if (command == DELETE) {
@@ -389,10 +411,42 @@ class SparkPositionDeltaWrite implements DeltaWrite, RequiresDistributionAndOrde
 
       return partitionProjections;
     }
+
+    // use a fanout writer only if enabled and the input is unordered and the table is partitioned
+    protected PartitioningWriter<InternalRow, DataWriteResult> newDataWriter(
+        Table table, SparkFileWriterFactory writers, OutputFileFactory files, Context context) {
+
+      FileIO io = table.io();
+      boolean useFanoutWriter = context.useFanoutWriter();
+      long targetFileSize = context.targetDataFileSize();
+
+      if (table.spec().isPartitioned() && useFanoutWriter) {
+        return new FanoutDataWriter<>(writers, files, io, targetFileSize);
+      } else {
+        return new ClusteredDataWriter<>(writers, files, io, targetFileSize);
+      }
+    }
+
+    // the spec requires position deletes to be ordered by file and position
+    // use a fanout writer if the input is unordered no matter whether fanout writers are enabled
+    // clustered writers assume that the position deletes are already ordered by file and position
+    protected PartitioningWriter<PositionDelete<InternalRow>, DeleteWriteResult> newDeleteWriter(
+        Table table, SparkFileWriterFactory writers, OutputFileFactory files, Context context) {
+
+      FileIO io = table.io();
+      boolean inputOrdered = context.inputOrdered();
+      long targetFileSize = context.targetDeleteFileSize();
+
+      if (inputOrdered) {
+        return new ClusteredPositionDeleteWriter<>(writers, files, io, targetFileSize);
+      } else {
+        return new FanoutPositionOnlyDeleteWriter<>(writers, files, io, targetFileSize);
+      }
+    }
   }
 
   private static class DeleteOnlyDeltaWriter extends BaseDeltaWriter {
-    private final ClusteredPositionDeleteWriter<InternalRow> delegate;
+    private final PartitioningWriter<PositionDelete<InternalRow>, DeleteWriteResult> delegate;
     private final PositionDelete<InternalRow> positionDelete;
     private final FileIO io;
     private final Map<Integer, PartitionSpec> specs;
@@ -411,9 +465,7 @@ class SparkPositionDeltaWrite implements DeltaWrite, RequiresDistributionAndOrde
         OutputFileFactory deleteFileFactory,
         Context context) {
 
-      this.delegate =
-          new ClusteredPositionDeleteWriter<>(
-              writerFactory, deleteFileFactory, table.io(), context.targetDeleteFileSize());
+      this.delegate = newDeleteWriter(table, writerFactory, deleteFileFactory, context);
       this.positionDelete = PositionDelete.create();
       this.io = table.io();
       this.specs = table.specs();
@@ -561,18 +613,6 @@ class SparkPositionDeltaWrite implements DeltaWrite, RequiresDistributionAndOrde
         this.closed = true;
       }
     }
-
-    private PartitioningWriter<InternalRow, DataWriteResult> newDataWriter(
-        Table table, SparkFileWriterFactory writers, OutputFileFactory files, Context context) {
-      long fileSize = context.targetDataFileSize();
-      return new ClusteredDataWriter<>(writers, files, table.io(), fileSize);
-    }
-
-    private ClusteredPositionDeleteWriter<InternalRow> newDeleteWriter(
-        Table table, SparkFileWriterFactory writers, OutputFileFactory files, Context context) {
-      long fileSize = context.targetDeleteFileSize();
-      return new ClusteredPositionDeleteWriter<>(writers, files, table.io(), fileSize);
-    }
   }
 
   private static class UnpartitionedDeltaWriter extends DeleteAndDataDeltaWriter {
@@ -640,8 +680,14 @@ class SparkPositionDeltaWrite implements DeltaWrite, RequiresDistributionAndOrde
     private final FileFormat deleteFileFormat;
     private final long targetDeleteFileSize;
     private final String queryId;
+    private final boolean useFanoutWriter;
+    private final boolean inputOrdered;
 
-    Context(Schema dataSchema, SparkWriteConf writeConf, LogicalWriteInfo info) {
+    Context(
+        Schema dataSchema,
+        SparkWriteConf writeConf,
+        LogicalWriteInfo info,
+        SparkWriteRequirements writeRequirements) {
       this.dataSchema = dataSchema;
       this.dataSparkType = info.schema();
       this.dataFileFormat = writeConf.dataFileFormat();
@@ -651,6 +697,8 @@ class SparkPositionDeltaWrite implements DeltaWrite, RequiresDistributionAndOrde
       this.targetDeleteFileSize = writeConf.targetDeleteFileSize();
       this.metadataSparkType = info.metadataSchema().get();
       this.queryId = info.queryId();
+      this.useFanoutWriter = writeConf.useFanoutWriter(writeRequirements);
+      this.inputOrdered = writeRequirements.hasOrdering();
     }
 
     Schema dataSchema() {
@@ -685,6 +733,14 @@ class SparkPositionDeltaWrite implements DeltaWrite, RequiresDistributionAndOrde
       return queryId;
     }
 
+    boolean useFanoutWriter() {
+      return useFanoutWriter;
+    }
+
+    boolean inputOrdered() {
+      return inputOrdered;
+    }
+
     int specIdOrdinal() {
       return metadataSparkType.fieldIndex(MetadataColumns.SPEC_ID.name());
     }
